@@ -4,7 +4,6 @@
 #include <glib-object.h>
 #include <glib/gprintf.h>
 
-
 #include "config_file.h"
 #include "status_file.h"
 #include "bundle.h"
@@ -23,6 +22,12 @@ typedef enum {
 	R_CONTEXT_CONFIG_MODE_REQUIRED, /* require config file */
 } RContextConfigMode;
 
+typedef struct {
+	gchar *section;
+	gchar *name;
+	gchar *value;
+} ConfigFileOverride;
+
 typedef struct {
 	/* a busy context must not be reconfigured */
 	gboolean busy;
@@ -31,6 +36,7 @@ typedef struct {
 	/* system configuration data */
 	RContextConfigMode configmode;
 	gchar *configpath;
+	GList *configoverride;
 	RaucConfig *config;
 
 	/* system status (not available when using per-slot status file) */
