@@ -12,6 +12,7 @@ typedef enum {
 	R_UTILS_ERROR_INAPPROPRIATE_IOCTL,
 	R_UTILS_ERROR_INVALID_ENV_KEY,
 	R_UTILS_ERROR_SEMVER_PARSE,
+	R_UTILS_ERROR_OPEN_FILE,
 } RUtilsError;
 
 #define BIT(nr) (1UL << (nr))
@@ -130,6 +131,17 @@ gchar *r_ptr_array_env_to_shell(const GPtrArray *ptrarray);
 gchar **r_environ_setenv_ptr_array(gchar **envp, const GPtrArray *ptrarray, gboolean overwrite)
 G_GNUC_WARN_UNUSED_RESULT;
 
+/**
+ * Calls g_subprocess_launcher_setenv for each 'key=value' string in the array.
+ *
+ * This is useful when setting up the environment via a subprocess launcher.
+ *
+ * @param launcher a GSubprocessLauncher
+ * @param ptrarray the GPtrArray to add to the environment
+ * @param overwrite whether to change existing variables
+ */
+void r_subprocess_launcher_setenv_ptr_array(GSubprocessLauncher *launcher, const GPtrArray *ptrarray, gboolean overwrite);
+
 /**
  * Read file content into a GBytes.
  *
@@ -189,6 +201,17 @@ G_GNUC_WARN_UNUSED_RESULT;
  */
 gboolean rm_tree(const gchar *path, GError **error);
 
+/**
+ * Recursively check directory tree for open files.
+ *
+ * @param path path of directory to check
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE if no files are open, FALSE otherwise
+ */
+gboolean r_tree_check_open(const gchar *path, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Resolve path based on directory of `basefile` argument or current working dir.
  *
