@@ -32,7 +32,7 @@ gboolean r_boot_is_supported_bootloader(const gchar *bootloader)
 	return g_strv_contains(supported_bootloaders, bootloader);
 }
 
-static GString *bootchooser_order_primay(RaucSlot *slot)
+static GString *bootchooser_order_primary(RaucSlot *slot)
 {
 	GString *order = NULL;
 	g_autoptr(GList) slots = NULL;
@@ -156,7 +156,6 @@ static gboolean barebox_state_get(const gchar* bootname, BareboxSlotState *bb_st
 	return TRUE;
 }
 
-
 /* names: list of gchar, values: list of gint */
 static gboolean barebox_state_set(GPtrArray *pairs, GError **error)
 {
@@ -517,7 +516,7 @@ static gboolean grub_get_state(RaucSlot* slot, gboolean *good, GError **error)
 
 	/* Check slot state */
 	key = g_strdup_printf("%s_OK", slot->bootname);
-	if (!grub_env_get(key, &slot_ok,  &ierror)) {
+	if (!grub_env_get(key, &slot_ok, &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
 	}
@@ -645,7 +644,7 @@ static gboolean grub_set_primary(RaucSlot *slot, GError **error)
 	g_return_val_if_fail(slot, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	order = bootchooser_order_primay(slot);
+	order = bootchooser_order_primary(slot);
 
 	g_ptr_array_add(pairs, g_strdup_printf("%s_OK=%i", slot->bootname, 1));
 	g_ptr_array_add(pairs, g_strdup_printf("%s_TRY=%i", slot->bootname, 0));
@@ -938,7 +937,7 @@ static gboolean uboot_set_primary(RaucSlot *slot, GError **error)
 		g_message("Unable to obtain BOOT_ORDER (%s), using defaults", ierror->message);
 		g_clear_error(&ierror);
 
-		order_current = bootchooser_order_primay(slot);
+		order_current = bootchooser_order_primary(slot);
 	}
 
 	/* Iterate over current boot order */
@@ -1013,7 +1012,6 @@ static gboolean efi_bootorder_set(gchar *order, GError **error)
 		return FALSE;
 	}
 
-
 	if (!g_subprocess_wait_check(sub, NULL, &ierror)) {
 		g_propagate_prefixed_error(
 				error,
@@ -1044,7 +1042,6 @@ static gboolean efi_set_bootnext(gchar *bootnumber, GError **error)
 		return FALSE;
 	}
 
-
 	if (!g_subprocess_wait_check(sub, NULL, &ierror)) {
 		g_propagate_prefixed_error(
 				error,
@@ -1590,7 +1587,7 @@ static RaucSlot* custom_get_primary(GError **error)
 
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	if (!custom_backend_get("get-primary", NULL, &ret_str, &ierror) ) {
+	if (!custom_backend_get("get-primary", NULL, &ret_str, &ierror)) {
 		g_propagate_error(error, ierror);
 		return NULL;
 	}
@@ -1657,7 +1654,6 @@ static gboolean custom_set_primary(RaucSlot *slot, GError **error)
 	return custom_backend_set("set-primary", slot->bootname, NULL, error);
 }
 
-
 gboolean r_boot_get_state(RaucSlot* slot, gboolean *good, GError **error)
 {
 	gboolean res = FALSE;
