@@ -5,6 +5,7 @@
 #include <glib/gstdio.h>
 #include <string.h>
 #include <fcntl.h>
+#include <sys/stat.h>
 #include <sys/vfs.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -15,6 +16,7 @@
 #include "bundle.h"
 #include "context.h"
 #include "crypt.h"
+#include "manifest.h"
 #include "mount.h"
 #include "signature.h"
 #include "utils.h"
@@ -104,7 +106,7 @@ r_bundle_error_quark(void)
 	return g_quark_from_static_string("r-bundle-error-quark");
 }
 
-static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GError **error)
+static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, const gchar *fakeroot, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -121,14 +123,18 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GEr
 		goto out;
 	}
 
+	r_fakeroot_add_args(args, fakeroot);
+
 	g_ptr_array_add(args, g_strdup("mksquashfs"));
 	g_ptr_array_add(args, g_strdup(contentdir));
 	g_ptr_array_add(args, g_strdup(bundlename));
-	g_ptr_array_add(args, g_strdup("-all-root"));
 	g_ptr_array_add(args, g_strdup("-noappend"));
 	g_ptr_array_add(args, g_strdup("-no-progress"));
-	g_ptr_array_add(args, g_strdup("-no-xattrs"));
-
+	if (!fakeroot) {
+		g_ptr_array_add(args, g_strdup("-all-root"));
+		g_ptr_array_add(args, g_strdup("-no-xattrs"));
+	}
+	g_ptr_array_add(args, g_strdup("-quiet"));
 
 	if (r_context()->mksquashfs_args != NULL) {
 		g_auto(GStrv) mksquashfs_argvp = NULL;
@@ -144,7 +150,7 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GEr
 	}
 	g_ptr_array_add(args, NULL);
 
-	res = r_subprocess_runv(args, G_SUBPROCESS_FLAGS_STDOUT_SILENCE, &ierror);
+	res = r_subprocess_runv(args, G_SUBPROCESS_FLAGS_NONE, &ierror);
 	if (!res) {
 		g_propagate_prefixed_error(
 				error,
@@ -196,6 +202,18 @@ out:
 	return res;
 }
 
+static gboolean check_pseudo_active(void)
+{
+	if (!g_getenv("PSEUDO_LOCALSTATEDIR"))
+		return FALSE;
+
+	const gchar *tmp = g_getenv("PSEUDO_DISABLED");
+	if (g_strcmp0(tmp, "1") == 0)
+		return FALSE;
+
+	return TRUE;
+}
+
 static gboolean casync_make_blob(const gchar *idxpath, const gchar *contentpath, const gchar *store, GError **error);
 
 static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath, const gchar *store, GError **error)
@@ -256,7 +274,9 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 	g_ptr_array_add(iargs, NULL);
 
 	/* Outer process calll */
-	g_ptr_array_add(args, g_strdup("fakeroot"));
+	if (!check_pseudo_active()) {
+		g_ptr_array_add(args, g_strdup("fakeroot"));
+	}
 	g_ptr_array_add(args, g_strdup("sh"));
 	g_ptr_array_add(args, g_strdup("-c"));
 	g_ptr_array_add(args, g_strjoinv(" ", (gchar**) iargs->pdata));
@@ -397,7 +417,7 @@ static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir,
 				g_autofree gchar *indexname = g_strconcat(image->filename, ".block-hash-index", NULL);
 				g_autofree gchar *indexpath = g_build_filename(dir, indexname, NULL);
 				g_autoptr(RaucHashIndex) index = NULL;
-				int fd = -1;
+				g_auto(filedesc) fd = -1;
 
 				if (image_is_archive(image)) {
 					g_warning("Generating block hash index requires a block device image but %s looks like an archive", image->filename);
@@ -420,7 +440,6 @@ static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir,
 							error,
 							ierror,
 							"Failed to generate hash index for %s: ", image->filename);
-					g_close(fd, NULL);
 					return FALSE;
 				}
 
@@ -429,7 +448,6 @@ static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir,
 							error,
 							ierror,
 							"Failed to write hash index for %s: ", image->filename);
-					g_close(fd, NULL);
 					return FALSE;
 				}
 
@@ -450,6 +468,115 @@ static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir,
 	return TRUE;
 }
 
+static gchar *convert_tar_extract(RaucImage *image, const gchar *dir, const gchar *fakeroot, GError **error)
+{
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(image, NULL);
+	g_return_val_if_fail(dir, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	g_autofree gchar *file_path = g_build_filename(dir, image->filename, NULL);
+	g_autofree gchar *converted = g_strdup_printf("%s.extracted", image->filename);
+	g_autofree gchar *converted_path = g_build_filename(dir, converted, NULL);
+
+	if (g_mkdir(converted_path, S_IRWXU)) {
+		int err = errno;
+		g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+				"failed to create directory '%s': %s", converted_path, g_strerror(err));
+		return NULL;
+	}
+
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(10, g_free);
+	r_fakeroot_add_args(args, fakeroot);
+
+	g_ptr_array_add(args, g_strdup("tar"));
+	g_ptr_array_add(args, g_strdup("--numeric-owner"));
+	// TODO add support for --acl --selinux --xattrs via fakeroot or pseudo
+	g_ptr_array_add(args, g_strdup("-xf"));
+	g_ptr_array_add(args, g_strdup(file_path));
+	g_ptr_array_add(args, g_strdup("-C"));
+	g_ptr_array_add(args, g_strdup(converted_path));
+	g_ptr_array_add(args, NULL);
+
+	if (!r_subprocess_runv(args, G_SUBPROCESS_FLAGS_NONE, &ierror)) {
+		g_propagate_prefixed_error(error, ierror, "failed to run tar extract: ");
+		return NULL;
+	}
+
+	return g_steal_pointer(&converted);
+}
+
+static gboolean convert_images(RaucManifest *manifest, const gchar *dir, const gchar *fakeroot, GError **error)
+{
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(manifest, FALSE);
+	g_return_val_if_fail(dir, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	for (GList *elem = manifest->images; elem != NULL; elem = elem->next) {
+		RaucImage *image = elem->data;
+
+		if (image->converted && image->converted->len == 0) {
+			g_set_error(
+					error,
+					R_BUNDLE_ERROR,
+					R_BUNDLE_ERROR_PAYLOAD,
+					"Manifest already contains converted images for '%s'", image->slotclass);
+			return FALSE;
+		}
+
+		if (!image->convert)
+			continue;
+
+		gboolean keep = FALSE;
+		g_autoptr(GPtrArray) converted = g_ptr_array_new_with_free_func(g_free);
+		for (gchar **method = image->convert; *method != NULL; method++) {
+			gchar *converted_filename = NULL;
+			if (g_str_equal(*method, "tar-extract")) {
+				g_debug("extracting tar artifact image '%s'", image->filename);
+				converted_filename = convert_tar_extract(image, dir, fakeroot, &ierror);
+				if (!converted_filename) {
+					g_propagate_error(error, ierror);
+					return FALSE;
+				}
+			} else if (g_str_equal(*method, "keep")) {
+				g_debug("keeping input artifact image '%s'", image->filename);
+				keep = TRUE;
+				converted_filename = g_strdup(image->filename);
+			} else {
+				g_set_error(
+						error,
+						R_BUNDLE_ERROR,
+						R_BUNDLE_ERROR_PAYLOAD,
+						"Unsupported convert method: %s", *method);
+				return FALSE;
+			}
+
+			g_assert(converted_filename != NULL);
+			g_ptr_array_add(converted, converted_filename);
+		}
+
+		if (!keep) {
+			g_autofree gchar *file_path = g_build_filename(dir, image->filename, NULL);
+			g_debug("removing input artifact image '%s' after conversion", image->filename);
+			if (g_unlink(file_path) != 0) {
+				int err = errno;
+				g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+						"Failed to remove input artifact image after conversion '%s': %s\n",
+						file_path, g_strerror(errno));
+				return FALSE;
+			}
+		}
+
+		g_clear_pointer(&image->converted, g_ptr_array_unref);
+		image->converted = g_steal_pointer(&converted);
+	}
+
+	return TRUE;
+}
+
 static gboolean output_stream_write_uint64_all(GOutputStream *stream,
 		guint64 data,
 		GCancellable *cancellable,
@@ -726,7 +853,6 @@ static GBytes *generate_bundle_signature(const gchar *bundlename, RaucManifest *
 	return g_steal_pointer(&sig);
 }
 
-
 static gboolean sign_bundle(const gchar *bundlename, RaucManifest *manifest, GError **error)
 {
 	GError *ierror = NULL;
@@ -877,12 +1003,68 @@ static gboolean decrypt_bundle_payload(RaucBundle *bundle, GError **error)
 out:
 	/* Remove temporary bundle creation directory.
 	 * RAUC will still have access to the decrypted bundle via the opened GFile */
-	if (tmpdir)
-		rm_tree(tmpdir, NULL);
+	if (tmpdir) {
+		g_autoptr(GError) tmperror = NULL;
+		if (!rm_tree(tmpdir, &tmperror)) {
+			g_warning("Failed to remove bundle decryption tmpdir %s: %s", tmpdir, tmperror->message);
+		}
+	}
 
 	return res;
 }
 
+static gboolean check_workdir_content(const gchar *path, GError **error)
+{
+	GError *ierror = NULL;
+	g_autofree gchar *name = g_path_get_basename(path);
+
+	/* It's not clear what absolute or relative symlinks (with slashes)
+	 * in the bundle input should mean, so reject them. If you have a use
+	 * case for this, contact us. */
+	if (g_file_test(path, G_FILE_TEST_IS_SYMLINK)) {
+		g_autofree gchar *link_target = g_file_read_link(path, &ierror);
+		if (!link_target) {
+			g_propagate_prefixed_error(error, ierror,
+					"failed to read symlink in bundle contents (%s): ", name);
+			return FALSE;
+		}
+
+		if (g_path_is_absolute(link_target)) {
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+					"absolute symlinks are not supported as bundle contents (%s)", name);
+			return FALSE;
+		}
+
+		if (strchr(link_target, '/')) {
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+					"symlinks containing slashes are not supported as bundle contents (%s)", name);
+			return FALSE;
+		}
+
+		/* Local symlinks in the same directory (containing no slashes) seem
+		 * harmless and can be useful in special cases for backwards
+		 * compatibility, so allow them. */
+
+		return TRUE;
+	}
+
+	/* Directories should not be needed in the bundle input, so reject
+	 * them. If you have a use case for this, contact us. */
+	if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+				"directories are not supported as bundle contents (%s)", name);
+		return FALSE;
+	}
+
+	if (!g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
+				"only regular files are supported as bundle contents (%s)", name);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gchar *prepare_workdir(const gchar *contentdir, GError **error)
 {
 	GError *ierror = NULL;
@@ -931,25 +1113,8 @@ static gchar *prepare_workdir(const gchar *contentdir, GError **error)
 
 		oldpath = g_build_filename(contentdir, name, NULL);
 
-		/* It's not clear what symlinks in the bundle input should mean, so
-		 * reject them. If you have a use case for this, contact us. */
-		if (g_file_test(oldpath, G_FILE_TEST_IS_SYMLINK)) {
-			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
-					"symlinks are not supported as bundle contents (%s)", name);
-			return NULL;
-		}
-
-		/* Directories should not be needed in the bundle input, so reject
-		 * them. If you have a use case for this, contact us. */
-		if (g_file_test(oldpath, G_FILE_TEST_IS_DIR)) {
-			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
-					"directories are not supported as bundle contents (%s)", name);
-			return NULL;
-		}
-
-		if (!g_file_test(oldpath, G_FILE_TEST_IS_REGULAR)) {
-			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_PAYLOAD,
-					"only regular files are supported as bundle contents (%s)", name);
+		if (!check_workdir_content(oldpath, &ierror)) {
+			g_propagate_error(error, ierror);
 			return NULL;
 		}
 
@@ -966,12 +1131,31 @@ static gchar *prepare_workdir(const gchar *contentdir, GError **error)
 	return g_steal_pointer(&workdir);
 }
 
+static gboolean needs_fakeroot(const RaucManifest *manifest)
+{
+	/* If pseudo is active, we don't need to use fakeroot. */
+	if (check_pseudo_active())
+		return FALSE;
+
+	for (GList *elem = manifest->images; elem != NULL; elem = elem->next) {
+		RaucImage *image = elem->data;
+
+		if (image->convert) {
+			if (g_strv_contains((const gchar * const *)image->convert, "tar-extract"))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError **error)
 {
 	GError *ierror = NULL;
 	g_autofree gchar* manifestpath = NULL;
 	g_autoptr(RaucManifest) manifest = NULL;
 	g_autofree gchar *workdir = NULL;
+	g_autofree gchar *fakeroot = NULL;
 	gboolean res = FALSE;
 
 	g_return_val_if_fail(bundlename != NULL, FALSE);
@@ -999,30 +1183,56 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 
 	g_print("Creating '%s' format bundle\n", r_manifest_bundle_format_to_str(manifest->bundle_format));
 
+	res = check_manifest_input(manifest, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	/* print warnings collected while parsing */
 	for (guint i = 0; i < manifest->warnings->len; i++) {
 		g_print("%s\n", (gchar *)g_ptr_array_index(manifest->warnings, i));
 	}
 
+	res = check_manifest_create(manifest, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	res = sync_manifest_with_contentdir(manifest, workdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
 
+	if (needs_fakeroot(manifest)) {
+		fakeroot = r_fakeroot_init(&ierror);
+		if (!fakeroot) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
 	res = generate_adaptive_data(manifest, workdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
 
+	res = convert_images(manifest, workdir, fakeroot, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	res = save_manifest_file(manifestpath, manifest, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
 
-	res = mksquashfs(bundlename, workdir, &ierror);
+	res = mksquashfs(bundlename, workdir, fakeroot, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1050,6 +1260,13 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 	res = TRUE;
 
 out:
+	/* Remove fakeroot env */
+	if (fakeroot) {
+		g_autofree GError *cleanup_error = NULL;
+		if (!r_fakeroot_cleanup(fakeroot, &cleanup_error)) {
+			g_warning("failed to clean up fakeroot environment: %s", cleanup_error->message);
+		}
+	}
 	/* Remove output file on error */
 	if (!res &&
 	    g_file_test(bundlename, G_FILE_TEST_IS_REGULAR))
@@ -1296,7 +1513,7 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 		goto out;
 	}
 
-	res = mksquashfs(outbundle, contentdir, &ierror);
+	res = mksquashfs(outbundle, contentdir, NULL, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1311,8 +1528,12 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 	res = TRUE;
 out:
 	/* Remove temporary bundle creation directory */
-	if (tmpdir)
-		rm_tree(tmpdir, NULL);
+	if (tmpdir) {
+		g_autoptr(GError) tmperror = NULL;
+		if (!rm_tree(tmpdir, &tmperror)) {
+			g_warning("Failed to remove conversion tmpdir %s: %s", tmpdir, tmperror->message);
+		}
+	}
 	return res;
 }
 
@@ -1416,6 +1637,7 @@ static gboolean is_remote_scheme(const gchar *scheme)
 {
 	return (g_strcmp0(scheme, "http") == 0) ||
 	       (g_strcmp0(scheme, "https") == 0) ||
+	       (g_strcmp0(scheme, "tftp") == 0) ||
 	       (g_strcmp0(scheme, "sftp") == 0) ||
 	       (g_strcmp0(scheme, "ftp") == 0) ||
 	       (g_strcmp0(scheme, "ftps") == 0);
@@ -2045,7 +2267,10 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, CheckBundleP
 		}
 		g_debug("Downloaded temp bundle to %s", ibundle->path);
 #else
-		g_warning("Mounting remote bundle not supported, recompile with -Dnetwork=true");
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSUPPORTED,
+				"Remote bundle access not supported, recompile with -Dstreaming=true");
+		res = FALSE;
+		goto out;
 #endif
 	} else {
 		ibundle->path = g_strdup(bundlename);
@@ -2530,8 +2755,12 @@ gboolean load_manifest_from_bundle(RaucBundle *bundle, RaucManifest **manifest,
 		goto out;
 	}
 out:
-	if (tmpdir)
-		rm_tree(tmpdir, NULL);
+	if (tmpdir) {
+		g_autoptr(GError) tmperror = NULL;
+		if (!rm_tree(tmpdir, &tmperror)) {
+			g_warning("Failed to remove unsquashfs tmpdir %s: %s", tmpdir, tmperror->message);
+		}
+	}
 	return res;
 }
 
