@@ -1,5 +1,6 @@
 #include <errno.h>
 #include <gio/gio.h>
+#include <glib/gstdio.h>
 #include <string.h>
 
 #include "config_file.h"
@@ -315,7 +316,7 @@ static GHashTable *get_system_info_from_handler(GError **error)
 	gchar *value = NULL;
 
 	if (!g_file_test(context->config->systeminfo_handler, G_FILE_TEST_EXISTS)) {
-		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,  "System info handler script/binary '%s' not found.", context->config->systeminfo_handler);
+		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT, "System info handler script/binary '%s' not found.", context->config->systeminfo_handler);
 		return NULL;
 	}
 
@@ -432,6 +433,28 @@ static gboolean r_context_configure_target(GError **error)
 	return TRUE;
 }
 
+static gboolean load_config_verbose(const char *configpath, GError **error)
+{
+	GError *ierror = NULL;
+
+	if (load_config(configpath, &context->config, &ierror)) {
+		g_message("Using system config file %s", configpath);
+		if (!context->configpath)
+			context->configpath = g_strdup(configpath);
+		return TRUE;
+	}
+
+	g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
+	return FALSE;
+}
+
+static const gchar *const search_paths[] = {
+	"/etc/rauc/system.conf",
+	"/run/rauc/system.conf",
+	"/usr/lib/rauc/system.conf",
+	NULL,
+};
+
 gboolean r_context_configure(GError **error)
 {
 	GError *ierror = NULL;
@@ -449,36 +472,35 @@ gboolean r_context_configure(GError **error)
 		/* explicitly set on the command line */
 		configmode = R_CONTEXT_CONFIG_MODE_REQUIRED;
 		configpath = context->configpath;
-	} else {
-		/* the default path */
-		configpath = "/etc/rauc/system.conf";
+	} else if (configmode != R_CONTEXT_CONFIG_MODE_NONE) {
+		for (const gchar *const *path = search_paths; *path; path++) {
+			GStatBuf st_buf;
+
+			configpath = *path;
+			if (g_stat(configpath, &st_buf) == 0)
+				break;
+		}
 	}
 	switch (configmode) {
+		case R_CONTEXT_CONFIG_MODE_REQUIRED:
+			if (load_config_verbose(configpath, error))
+				break;
+			return FALSE;
+		case R_CONTEXT_CONFIG_MODE_AUTO:
+			if (load_config_verbose(configpath, &ierror))
+				break;
+			if (ierror->domain != G_FILE_ERROR &&
+			    !g_error_matches(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_EMPTY_FILE)) {
+				g_propagate_error(error, ierror);
+				return FALSE;
+			}
+
+			g_clear_error(&ierror);
+		/* This is a hack as we cannot get rid of config easily */
+		/* Fallthrough */
 		case R_CONTEXT_CONFIG_MODE_NONE:
 			default_config(&context->config);
 			break;
-		case R_CONTEXT_CONFIG_MODE_AUTO:
-			if (load_config(configpath, &context->config, &ierror)) {
-				g_message("valid %s found, using it", configpath);
-				if (!context->configpath)
-					context->configpath = g_strdup(configpath);
-			} else if (ierror->domain != G_FILE_ERROR) {
-				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
-				return FALSE;
-			} else {
-				g_clear_error(&ierror);
-				/* This is a hack as we cannot get rid of config easily */
-				default_config(&context->config);
-			}
-			break;
-		case R_CONTEXT_CONFIG_MODE_REQUIRED:
-			if (!load_config(configpath, &context->config, &ierror)) {
-				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
-				return FALSE;
-			}
-			if (!context->configpath)
-				context->configpath = g_strdup(configpath);
-			break;
 		default:
 			g_error("invalid context config mode %d", configmode);
 			break;
@@ -871,6 +893,15 @@ void r_context_clean(void)
 
 		g_clear_pointer(&context->config, free_config);
 
+		for (GList *l = context->configoverride; l != NULL; l = l->next) {
+			ConfigFileOverride *override = (ConfigFileOverride *)l->data;
+			g_clear_pointer(&override->section, g_free);
+			g_clear_pointer(&override->name, g_free);
+			g_clear_pointer(&override->value, g_free);
+			g_clear_pointer(&override, g_free);
+		}
+		g_clear_pointer(&context->configoverride, g_list_free);
+
 		g_clear_pointer(&context->system_status, r_system_status_free);
 
 		g_clear_pointer(&context, g_free);
