@@ -12,6 +12,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "artifacts.h"
 #include "bootchooser.h"
 #include "bundle.h"
 #include "context.h"
@@ -21,6 +22,7 @@
 #include "mark.h"
 #include "mount.h"
 #include "service.h"
+#include "shell.h"
 #include "signature.h"
 #include "slot.h"
 #include "status_file.h"
@@ -80,7 +82,7 @@ static gchar *resolve_loop_device(const gchar *devicepath, GError **error)
 		g_propagate_prefixed_error(
 				error,
 				ierror,
-				"Error getting loop backing_file for %s: ", devicepath);
+				"Error getting loop backing_file for '%s': ", devicepath);
 		return NULL;
 	}
 
@@ -123,7 +125,7 @@ gboolean update_external_mount_points(GError **error)
 				continue;
 			}
 			s->ext_mount_point = g_strdup(g_unix_mount_get_mount_path(m));
-			g_debug("Found external mountpoint for slot %s at %s", s->name, s->ext_mount_point);
+			g_debug("Found external mountpoint for slot '%s' at '%s'", s->name, s->ext_mount_point);
 		}
 	}
 
@@ -243,7 +245,7 @@ gboolean determine_slot_states(GError **error)
 
 		if (s == booted) {
 			s->state = ST_BOOTED;
-			g_debug("Found booted slot: %s on %s", s->name, s->device);
+			g_debug("Found booted slot: '%s' on '%s'", s->name, s->device);
 		} else if (s->parent && s->parent == booted) {
 			s->state = ST_ACTIVE;
 		} else {
@@ -271,7 +273,7 @@ gboolean determine_boot_states(GError **error)
 			continue;
 
 		if (!r_boot_get_state(slot, &slot->boot_good, &ierror)) {
-			g_message("Failed to get boot state of %s: %s", slot->name, ierror->message);
+			g_message("Failed to get boot state of '%s': %s", slot->name, ierror->message);
 			had_errors = TRUE;
 		}
 	}
@@ -301,6 +303,10 @@ static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
 		const gchar *key = NULL;
 		RaucImage *iterimage = l->data;
 		g_assert_nonnull(iterimage->slotclass);
+
+		if (iterimage->artifact)
+			continue;
+
 		key = g_intern_string(iterimage->slotclass);
 		g_ptr_array_remove_fast(slotclasses, (gpointer)key); /* avoid duplicates */
 		g_ptr_array_add(slotclasses, (gpointer)key);
@@ -309,6 +315,7 @@ static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
 
 	return (gchar**) g_ptr_array_free(slotclasses, FALSE);
 }
+
 /* Selects a single appropriate inactive slot of root slot class
  *
  * Note: This function may be extended to be more sophisticated or follow a
@@ -382,11 +389,10 @@ GHashTable* determine_target_install_group(void)
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &iterslot)) {
 		RaucSlot *parent = r_slot_get_parent_root(iterslot);
-		g_debug("Checking slot: %s", iterslot->name);
-
+		g_debug("Checking slot: '%s'", iterslot->name);
 
 		if (r_slot_list_contains(selected_root_slots, parent)) {
-			g_debug("\tAdding mapping: %s -> %s", iterslot->sclass, iterslot->name);
+			g_debug("\tAdding mapping: '%s' -> '%s'", iterslot->sclass, iterslot->name);
 			g_hash_table_insert(targetgroup, (gpointer) iterslot->sclass, iterslot);
 		} else {
 			g_debug("\tNo mapping found");
@@ -422,7 +428,7 @@ GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target
 
 	slotclasses = get_all_manifest_slot_classes(manifest);
 
-	/* Find exactly 1 image for each class listed in manifest */
+	/* Find exactly 1 image for each slot class listed in manifest */
 	for (gchar **cls = slotclasses; *cls != NULL; cls++) {
 		RaucImage *matching_img = NULL;
 		g_autoptr(RImageInstallPlan) plan = g_new0(RImageInstallPlan, 1);
@@ -442,7 +448,7 @@ GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target
 				if (!matching_img)
 					matching_img = lookup_image;
 			} else if (g_strcmp0(lookup_image->variant, r_context()->config->system_variant) == 0) {
-				g_debug("Using variant %s image %s for %s", lookup_image->variant, lookup_image->filename, lookup_image->slotclass);
+				g_debug("Using variant '%s' image '%s' for '%s'", lookup_image->variant, lookup_image->filename, lookup_image->slotclass);
 				matching_img = lookup_image;
 				break;
 			}
@@ -454,11 +460,11 @@ GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target
 			g_set_error(error,
 					R_INSTALL_ERROR,
 					R_INSTALL_ERROR_FAILED,
-					"Failed to find matching variant of image for %s", *cls);
+					"Failed to find matching variant of image for slot class '%s'", *cls);
 			return NULL;
 		}
 
-		g_debug("Found image mapping: %s -> %s", matching_img->filename, matching_img->slotclass);
+		g_debug("Found image mapping: '%s' -> '%s'", matching_img->filename, matching_img->slotclass);
 		plan->image = matching_img;
 
 		/* Check if target_group contains an appropriate slot for this image */
@@ -467,14 +473,14 @@ GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target
 			g_set_error(error,
 					R_INSTALL_ERROR,
 					R_INSTALL_ERROR_FAILED,
-					"No target slot for class %s of image %s found", matching_img->slotclass, matching_img->filename);
+					"No target slot for class '%s' of image '%s' found", matching_img->slotclass, matching_img->filename);
 			return NULL;
 		}
 		if (plan->target_slot->readonly) {
 			g_set_error(error,
 					R_INSTALL_ERROR,
 					R_INSTALL_ERROR_FAILED,
-					"Target slot for class %s of image %s is readonly", matching_img->slotclass, matching_img->filename);
+					"Target slot for class '%s' of image '%s' is readonly", matching_img->slotclass, matching_img->filename);
 			return NULL;
 		}
 
@@ -488,6 +494,37 @@ GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target
 		g_ptr_array_add(install_plans, g_steal_pointer(&plan));
 	}
 
+	/* Make plans for artifact installation */
+	if (!r_artifacts_init(&ierror)) {
+		g_propagate_error(error, ierror);
+		return NULL;
+	}
+	/* TODO: This may remove objects for a composefs image we are about to install. */
+	if (!r_artifacts_prune(&ierror)) {
+		g_propagate_error(error, ierror);
+		return NULL;
+	}
+
+	for (GList *l = manifest->images; l != NULL; l = l->next) {
+		RaucImage *iter_image = l->data;
+
+		if (!iter_image->artifact)
+			continue;
+
+		g_autoptr(RImageInstallPlan) plan = g_new0(RImageInstallPlan, 1);
+		plan->image = l->data;
+		plan->target_repo = g_hash_table_lookup(r_context()->config->artifact_repos, iter_image->slotclass);
+		if (!plan->target_repo) {
+			g_set_error(error,
+					R_INSTALL_ERROR,
+					R_INSTALL_ERROR_FAILED,
+					"No target repo '%s' for artifact image '%s' found", iter_image->slotclass, iter_image->artifact);
+			return NULL;
+		}
+
+		g_ptr_array_add(install_plans, g_steal_pointer(&plan));
+	}
+
 	if (install_plans->len == 0) {
 		g_set_error_literal(error,
 				R_INSTALL_ERROR,
@@ -603,6 +640,7 @@ static gchar **add_system_environment(gchar **envp)
 	g_return_val_if_fail(envp, NULL);
 
 	envp = g_environ_setenv(envp, "RAUC_SYSTEM_CONFIG", r_context()->configpath, TRUE);
+	envp = g_environ_setenv(envp, "RAUC_SYSTEM_VARIANT", r_context()->config->system_variant ?: "", TRUE);
 	envp = g_environ_setenv(envp, "RAUC_CURRENT_BOOTNAME", r_context()->bootslot, TRUE);
 	envp = g_environ_setenv(envp, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
 
@@ -620,7 +658,7 @@ static gchar **add_system_environment(gchar **envp)
 /**
  * Sets up an environment containing RAUC information, ready to be passed to e.g. handlers
  *
- * Extends the system environment so that the result is save to be used with
+ * Extends the system environment so that the result is safe to be used with
  * g_subprocess_launcher_set_environ().
  *
  * @param update_source Path to the current bundle mount point
@@ -716,6 +754,10 @@ static gchar **prepare_environment(gchar *update_source, RaucManifest *manifest,
 	envp = g_environ_setenv(envp, "RAUC_SLOTS", slots->str, TRUE);
 	envp = g_environ_setenv(envp, "RAUC_TARGET_SLOTS", target_slots->str, TRUE);
 
+	g_autoptr(GPtrArray) shell_vars = g_ptr_array_new_with_free_func(g_free);
+	r_shell_from_manifest_meta(shell_vars, manifest);
+	envp = r_environ_setenv_ptr_array(envp, shell_vars, FALSE);
+
 	return envp;
 }
 
@@ -812,7 +854,7 @@ static gboolean run_bundle_hook(RaucManifest *manifest, gchar* bundledir, const
 
 	hook_name = g_build_filename(bundledir, manifest->hook_name, NULL);
 
-	g_message("Running bundle hook %s", hook_cmd);
+	g_message("Running bundle hook '%s'", hook_cmd);
 
 	launcher = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_STDERR_PIPE);
 
@@ -824,6 +866,10 @@ static gboolean run_bundle_hook(RaucManifest *manifest, gchar* bundledir, const
 	g_subprocess_launcher_setenv(launcher, "RAUC_MF_BUILD", manifest->update_build ?: "", TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
 
+	g_autoptr(GPtrArray) shell_vars = g_ptr_array_new_with_free_func(g_free);
+	r_shell_from_manifest_meta(shell_vars, manifest);
+	r_subprocess_launcher_setenv_ptr_array(launcher, shell_vars, TRUE);
+
 	sproc = g_subprocess_launcher_spawn(
 			launcher, &ierror,
 			hook_name,
@@ -856,7 +902,7 @@ static gboolean run_bundle_hook(RaucManifest *manifest, gchar* bundledir, const
 			if (hookreturnmsg) {
 				g_clear_error(&ierror);
 				g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED,
-						"Hook returned: %s", hookreturnmsg);
+						"Hook returned: '%s'", hookreturnmsg);
 			} else {
 				g_propagate_prefixed_error(
 						error,
@@ -960,20 +1006,56 @@ static gboolean pre_install_checks(gchar* bundledir, GPtrArray *install_plans, G
 		}
 
 		if (!g_file_test(plan->image->filename, G_FILE_TEST_EXISTS)) {
-			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
-					"Source image '%s' not found in bundle", plan->image->filename);
-			return FALSE;
+			if (plan->image->converted->len == 0) {
+				g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
+						"Source image '%s' not found in bundle", plan->image->filename);
+				return FALSE;
+			} else {
+				g_debug("Have converted images, ignoring missing source image '%s'", plan->image->filename);
+			}
 		}
 
+		/* handle converted images as well */
+		if (plan->image->converted)
+			for (guint j = 0; j < plan->image->converted->len; j++) {
+				gchar **converted = (gchar**)&plan->image->converted->pdata[j];
+				if (!g_path_is_absolute(*converted)) {
+					gchar *filename = g_build_filename(bundledir, *converted, NULL);
+					g_free(*converted);
+					*converted = filename;
+				}
+
+				if (!g_file_test(*converted, G_FILE_TEST_EXISTS)) {
+					g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
+							"Converted source image '%s' not found in bundle", *converted);
+					return FALSE;
+				}
+			}
+
 skip_filename_checks:
-		if (!g_file_test(plan->target_slot->device, G_FILE_TEST_EXISTS)) {
-			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
-					"Destination device '%s' not found", plan->target_slot->device);
-			return FALSE;
-		}
+		if (plan->target_slot) {
+			g_assert(plan->target_repo == NULL);
 
-		if (!pre_install_check_slot_mount_status(plan->target_slot, plan->image, error)) {
-			/* error is already set */
+			if (!g_file_test(plan->target_slot->device, G_FILE_TEST_EXISTS)) {
+				g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
+						"Destination device '%s' for slot '%s' not found", plan->target_slot->device, plan->target_slot->name);
+				return FALSE;
+			}
+
+			if (!pre_install_check_slot_mount_status(plan->target_slot, plan->image, error)) {
+				/* error is already set */
+				return FALSE;
+			}
+		} else if (plan->target_repo) {
+			g_assert(plan->target_slot == NULL);
+
+			if (!g_file_test(plan->target_repo->path, G_FILE_TEST_EXISTS)) {
+				g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
+						"Destination path '%s' not found for artifact repo '%s'", plan->target_repo->path, plan->target_repo->name);
+				return FALSE;
+			}
+		} else {
+			g_error("Inconsistency detected: image install plan has no target");
 			return FALSE;
 		}
 	}
@@ -1010,7 +1092,11 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 
 	install_args_update(args, "Checking slot %s", plan->target_slot->name);
 
-	r_context_begin_step_weighted_formatted("check_slot", 0, 1, "Checking slot %s", plan->target_slot->name);
+	r_context_begin_step_weighted_formatted("check_slot", 0, 1, "Checking slot %s%s%s%s",
+			plan->target_slot->name,
+			plan->target_slot->bootname ? " (" : "",
+			plan->target_slot->bootname ? plan->target_slot->bootname : "",
+			plan->target_slot->bootname ? ")" : "");
 
 	r_slot_status_load(plan->target_slot);
 	slot_state = plan->target_slot->status;
@@ -1025,9 +1111,9 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 	}
 
 	/* if explicitly enabled, skip update of up-to-date slots */
-	if (!plan->target_slot->install_same && g_strcmp0(plan->image->checksum.digest, slot_state->checksum.digest) == 0) {
-		install_args_update(args, "Skipping update for correct image %s", plan->image->filename);
-		g_message("Skipping update for correct image %s", plan->image->filename);
+	if (!plan->target_slot->install_same && g_strcmp0(slot_state->status, "ok") == 0 && g_strcmp0(plan->image->checksum.digest, slot_state->checksum.digest) == 0) {
+		install_args_update(args, "Skipping update for correct image '%s'", plan->image->filename);
+		g_message("Skipping update for correct image '%s'", plan->image->filename);
 		r_context_end_step("check_slot", TRUE);
 
 		/* Dummy step to indicate slot was skipped and complete required 'update_slots' substeps */
@@ -1128,10 +1214,14 @@ static RaucSlot* get_boot_mark_slot(const GPtrArray *install_plans)
 {
 	RaucSlot *bootslot = NULL;
 
+	g_return_val_if_fail(install_plans, NULL);
+
 	for (guint i = 0; i < install_plans->len; i++) {
 		const RImageInstallPlan *plan = g_ptr_array_index(install_plans, i);
 
-		g_assert_nonnull(plan->target_slot);
+		if (!plan->target_slot) {
+			continue;
+		}
 
 		if (plan->target_slot->parent || !plan->target_slot->bootname) {
 			continue;
@@ -1211,6 +1301,122 @@ static void log_event_installation_done(RaucInstallArgs *args, RaucManifest *man
 	g_log_structured_array(G_LOG_LEVEL_MESSAGE, fields, G_N_ELEMENTS(fields));
 }
 
+static gboolean remove_old_artifacts(const RaucManifest *manifest, RArtifactRepo *repo, GError **error)
+{
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(manifest, FALSE);
+	g_return_val_if_fail(repo, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	/* Remove references to old artifacts in this repo which are not in the manifest. */
+	/* The symlinks and data will be removed later when pruning. */
+	GHashTableIter iter;
+	g_hash_table_iter_init(&iter, repo->artifacts);
+	const gchar *a_name = NULL;
+	GHashTable *inner = NULL;
+	while (g_hash_table_iter_next(&iter, (gpointer*)&a_name, (gpointer*)&inner)) {
+		if (r_manifest_has_artifact_image(manifest, repo->name, a_name)) {
+			/* The manifest has an image for this artifact. */
+			continue;
+		}
+
+		GHashTableIter inner_iter;
+		g_hash_table_iter_init(&inner_iter, inner);
+		RArtifact *artifact = NULL;
+		while (g_hash_table_iter_next(&inner_iter, NULL, (gpointer*)&artifact)) {
+			g_assert(artifact->references != NULL);
+
+			g_message("Deactivating artifact '%s' in repo '%s', because it is not in the new manifest", a_name, repo->name);
+
+			/* TODO support repos with multiple parents, perhaps with r_artifact_deactivate */
+			g_ptr_array_set_size(artifact->references, 0);
+		}
+	}
+
+	if (!r_artifact_repo_commit(repo, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean handle_artifact_install_plan(const RaucManifest *manifest, const RImageInstallPlan *plan, RaucInstallArgs *args, const char *hook_name, GError **error)
+{
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(manifest, FALSE);
+	g_return_val_if_fail(plan, FALSE);
+	g_return_val_if_fail(args, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	r_context_begin_step_weighted_formatted("check_repo", 0, 1, "Checking repo '%s'", plan->target_repo->name);
+	if (!remove_old_artifacts(manifest, plan->target_repo, &ierror)) {
+		g_propagate_error(error, ierror);
+		r_context_end_step("check_repo", FALSE);
+		return FALSE;
+	}
+
+	/* check for existing artifact */
+	RArtifact *artifact = r_artifact_find(plan->target_repo, plan->image->artifact, plan->image->checksum.digest);
+	gboolean need_install = FALSE;
+	if (!artifact) {
+		/* create artifact */
+		g_autoptr(RArtifact) artifact_owned = g_new0(RArtifact, 1);
+		artifact = artifact_owned;
+
+		artifact->name = g_intern_string(plan->image->artifact);
+		artifact->bundle_compatible = g_strdup(manifest->update_compatible);
+		artifact->bundle_version = g_strdup(manifest->update_version);
+		artifact->bundle_description = g_strdup(manifest->update_description);
+		artifact->bundle_build = g_strdup(manifest->update_build);
+		artifact->bundle_hash = g_strdup(manifest->hash);
+		artifact->checksum.digest = g_strdup(plan->image->checksum.digest);
+		artifact->checksum.size = plan->image->checksum.size;
+		artifact->checksum.type = plan->image->checksum.type;
+		artifact->references = g_ptr_array_new();
+
+		/* find target name */
+		if (!r_artifact_repo_insert(plan->target_repo, artifact, &ierror)) {
+			g_propagate_error(error, ierror);
+			r_context_end_step("check_repo", FALSE);
+			return FALSE;
+		}
+		artifact_owned = NULL; /* it now belongs to the repo */
+		need_install = TRUE;
+	}
+	r_context_end_step("check_repo", TRUE);
+
+	r_event_log_message(R_EVENT_LOG_TYPE_WRITE_SLOT, "Updating artifact '%s' in repo '%s'", artifact->name, plan->target_repo->name);
+
+	if (need_install) {
+		r_context_begin_step_weighted_formatted("copy_image", 0, 9, "Copying artifact image to repo '%s'", plan->target_repo->name);
+
+		if (!r_artifact_install(artifact, plan->image, &ierror)) {
+			g_propagate_error(error, ierror);
+			r_context_end_step("copy_image", FALSE);
+			return FALSE;
+		}
+	} else {
+		r_context_begin_step_weighted_formatted("copy_image", 0, 9, "Reusing artifact image in repo '%s'", plan->target_repo->name);
+	}
+
+	/* update links (commit) */
+	g_assert(artifact->repo->parent_class == NULL); /* TODO handle repos with parents */
+	r_artifact_activate(artifact, (gpointer)g_intern_static_string(""));
+	if (!r_artifact_repo_commit(plan->target_repo, &ierror)) {
+		g_propagate_error(error, ierror);
+		r_context_end_step("copy_image", FALSE);
+		return FALSE;
+	}
+
+	r_context_end_step("copy_image", TRUE);
+
+	install_args_update(args, "Updating artifact '%s' in repo '%s' done", plan->image->artifact, plan->target_repo->name);
+	return TRUE;
+}
+
 static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bundledir, RaucManifest *manifest, GHashTable *target_group, GError **error)
 {
 	g_autofree gchar *hook_name = NULL;
@@ -1251,13 +1457,27 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	for (guint i = 0; i < install_plans->len; i++) {
 		const RImageInstallPlan *plan = g_ptr_array_index(install_plans, i);
 
-		if (!handle_slot_install_plan(manifest, plan, args, hook_name, &ierror)) {
-			g_propagate_error(error, ierror);
-			r_context_end_step("update_slots", FALSE);
-			return FALSE;
+		if (plan->target_slot) {
+			if (!handle_slot_install_plan(manifest, plan, args, hook_name, &ierror)) {
+				g_propagate_error(error, ierror);
+				r_context_end_step("update_slots", FALSE);
+				return FALSE;
+			}
+		} else if (plan->target_repo) {
+			if (!handle_artifact_install_plan(manifest, plan, args, hook_name, &ierror)) {
+				g_propagate_error(error, ierror);
+				r_context_end_step("update_slots", FALSE);
+				return FALSE;
+			}
 		}
 	}
 
+	/* Remove unused artifacts if we were successful so far. */
+	if (!r_artifacts_prune(&ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
 	r_context_end_step("update_slots", TRUE);
 	install_args_update(args, "All slots updated");
 
@@ -1275,7 +1495,6 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 		}
 	}
 
-
 	return TRUE;
 }
 
@@ -1494,7 +1713,6 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 		}
 	}
 
-
 	res = TRUE;
 
 umount:
@@ -1580,7 +1798,7 @@ gboolean install_run(RaucInstallArgs *args)
 	g_autoptr(GThread) thread = NULL;
 	r_context_set_busy(TRUE);
 
-	g_message("Active slot bootname: %s", r_context()->bootslot);
+	g_message("Active slot bootname: '%s'", r_context()->bootslot);
 
 	thread = g_thread_new("installer", install_thread, args);
 	if (thread == NULL)
