@@ -13,6 +13,7 @@
 #include <sys/ioctl.h>
 #include <unistd.h>
 
+#include "artifacts.h"
 #include "bundle.h"
 #include "bootchooser.h"
 #include "config_file.h"
@@ -21,7 +22,9 @@
 #include "install.h"
 #include "rauc-installer-generated.h"
 #include "service.h"
+#include "shell.h"
 #include "signature.h"
+#include "slot.h"
 #include "status_file.h"
 #include "update_handler.h"
 #include "utils.h"
@@ -338,7 +341,6 @@ static gboolean install_start(int argc, char **argv)
 
 	g_main_loop_run(r_loop);
 
-
 out_loop:
 	switch (args->status_result) {
 		case 0:
@@ -867,6 +869,29 @@ out:
 #define KWHT  "\x1B[37m"
 #define KBLD  "\x1B[1m"
 
+static void formatter_shell_append_idx_str(GPtrArray *entries, const gchar* varname, gint outer_idx, gint middle_idx, guint cnt)
+{
+	GString *text = g_string_new(varname);
+
+	if (outer_idx >= 0) {
+		g_string_append_printf(text, "_%d", outer_idx);
+	}
+
+	if (middle_idx >= 0) {
+		g_string_append_printf(text, "_%d", middle_idx);
+	}
+
+	g_string_append_c(text, '=');
+
+	for (guint i = 1; i <= cnt; i++) {
+		g_string_append_printf(text, "%d", i);
+		if (i < cnt)
+			g_string_append_c(text, ' ');
+	}
+
+	g_ptr_array_add(entries, g_string_free(text, FALSE));
+}
+
 static gchar *info_formatter_shell(RaucManifest *manifest)
 {
 	g_autoptr(GPtrArray) entries = g_ptr_array_new_with_free_func(g_free);
@@ -894,37 +919,14 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 
 	g_ptr_array_unref(hooks);
 
-	if (manifest->meta && g_hash_table_size(manifest->meta)) {
-		GHashTableIter iter;
-		GHashTable *kvs;
-		const gchar *group;
-
-		g_hash_table_iter_init(&iter, manifest->meta);
-		while (g_hash_table_iter_next(&iter, (gpointer*)&group, (gpointer*)&kvs)) {
-			GHashTableIter kvs_iter;
-			const gchar *key, *value;
-			g_autofree gchar *env_group = r_prepare_env_key(group, NULL);
-
-			if (!env_group)
-				continue;
-
-			g_hash_table_iter_init(&kvs_iter, kvs);
-			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
-				g_autofree gchar *env_key = r_prepare_env_key(key, NULL);
-
-				if (!env_key)
-					continue;
-
-				r_ptr_array_add_printf(entries, "RAUC_META_%s_%s=%s", env_group, env_key, value);
-			}
-		}
-	}
+	r_shell_from_manifest_meta(entries, manifest);
 
 	cnt = 0;
 	for (GList *l = manifest->images; l != NULL; l = l->next) {
 		RaucImage *img = l->data;
 		r_ptr_array_add_printf(entries, "RAUC_IMAGE_NAME_%d=%s", cnt, img->filename ?: "");
 		r_ptr_array_add_printf(entries, "RAUC_IMAGE_CLASS_%d=%s", cnt, img->slotclass ?: "");
+		r_ptr_array_add_printf(entries, "RAUC_IMAGE_ARTIFACT_%d=%s", cnt, img->artifact ?: "");
 		r_ptr_array_add_printf(entries, "RAUC_IMAGE_VARIANT_%d=%s", cnt, img->variant ?: "");
 		r_ptr_array_add_printf(entries, "RAUC_IMAGE_DIGEST_%d=%s", cnt, img->checksum.digest ?: "");
 		r_ptr_array_add_printf(entries, "RAUC_IMAGE_SIZE_%d=%"G_GOFFSET_FORMAT, cnt, img->checksum.size);
@@ -953,6 +955,22 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 			g_free(temp_string);
 		}
 
+		if (img->convert) {
+			temp_string = g_strjoinv(" ", (gchar**) img->convert);
+			r_ptr_array_add_printf(entries, "RAUC_IMAGE_CONVERT_%d=%s", cnt, temp_string);
+			g_free(temp_string);
+		}
+		if (img->converted) {
+			GString *temp_text = g_string_new(NULL);
+			g_string_append_printf(temp_text, "RAUC_IMAGE_CONVERTED_%d=", cnt);
+			for (guint i = 0; i < img->converted->len; i++) {
+				if (i)
+					g_string_append_c(temp_text, ' ');
+				g_string_append(temp_text, img->converted->pdata[i]);
+			}
+			g_ptr_array_add(entries, g_string_free(temp_text, FALSE));
+		}
+
 		cnt++;
 	}
 
@@ -967,10 +985,10 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 	gboolean show_crypt_key = FALSE; /* change to TRUE to display dm-crypt key */
 	gint cnt;
 
-	g_string_append_printf(text, "Compatible: \t'%s'\n", manifest->update_compatible);
-	g_string_append_printf(text, "Version:    \t'%s'\n", manifest->update_version);
-	g_string_append_printf(text, "Description:\t'%s'\n", manifest->update_description);
-	g_string_append_printf(text, "Build:      \t'%s'\n", manifest->update_build);
+	g_string_append_printf(text, "Compatible:     '%s'\n", manifest->update_compatible);
+	g_string_append_printf(text, "Version:        '%s'\n", manifest->update_version);
+	g_string_append_printf(text, "Description:    '%s'\n", manifest->update_description);
+	g_string_append_printf(text, "Build:          '%s'\n", manifest->update_build);
 
 	hooks = g_ptr_array_new();
 	if (manifest->hooks.install_check == TRUE) {
@@ -979,10 +997,10 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 	g_ptr_array_add(hooks, NULL);
 
 	temp_string = g_strjoinv(" ", (gchar**) hooks->pdata);
-	g_string_append_printf(text, "Hooks:      \t'%s'\n", temp_string);
+	g_string_append_printf(text, "Hooks:          '%s'\n", temp_string);
 	g_free(temp_string);
 
-	g_string_append_printf(text, "Bundle Format: \t%s", r_manifest_bundle_format_to_str(manifest->bundle_format));
+	g_string_append_printf(text, "Bundle Format:  %s", r_manifest_bundle_format_to_str(manifest->bundle_format));
 	if (manifest->bundle_format == R_MANIFEST_FORMAT_CRYPT) {
 		if (manifest->was_encrypted)
 			g_string_append_printf(text, KBLD KGRN " [encrypted CMS]"KNRM);
@@ -992,14 +1010,14 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 	g_string_append_printf(text, "\n");
 
 	if (manifest->bundle_format == R_MANIFEST_FORMAT_CRYPT) {
-		g_string_append_printf(text, "  Crypt Key: \t'%s'\n", show_crypt_key ? manifest->bundle_crypt_key : "<hidden>");
+		g_string_append_printf(text, "  Crypt Key:    '%s'\n", show_crypt_key ? manifest->bundle_crypt_key : "<hidden>");
 	}
 	if (manifest->bundle_format == R_MANIFEST_FORMAT_VERITY || manifest->bundle_format == R_MANIFEST_FORMAT_CRYPT) {
-		g_string_append_printf(text, "  Verity Salt: \t'%s'\n", manifest->bundle_verity_salt);
-		g_string_append_printf(text, "  Verity Hash: \t'%s'\n", manifest->bundle_verity_hash);
-		g_string_append_printf(text, "  Verity Size: \t%"G_GUINT64_FORMAT "\n", manifest->bundle_verity_size);
+		g_string_append_printf(text, "  Verity Salt:  '%s'\n", manifest->bundle_verity_salt);
+		g_string_append_printf(text, "  Verity Hash:  '%s'\n", manifest->bundle_verity_hash);
+		g_string_append_printf(text, "  Verity Size:  %"G_GUINT64_FORMAT "\n", manifest->bundle_verity_size);
 	}
-	g_string_append_printf(text, "Manifest Hash:\t'%s'\n\n", manifest->hash);
+	g_string_append_printf(text, "Manifest Hash:  '%s'\n\n", manifest->hash);
 
 	g_ptr_array_unref(hooks);
 
@@ -1015,11 +1033,11 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 			GHashTableIter kvs_iter;
 			const gchar *key, *value;
 
-			g_string_append_printf(text, "\t%s:\n", group);
+			g_string_append_printf(text, "  %s:\n", group);
 
 			g_hash_table_iter_init(&kvs_iter, kvs);
 			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
-				g_string_append_printf(text, "\t\t%s: %s\n", key, value);
+				g_string_append_printf(text, "    %s: %s\n", key, value);
 			}
 		}
 	}
@@ -1031,14 +1049,16 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		RaucImage *img = l->data;
 		g_string_append_printf(text, "  "KBLD "[%s]"KNRM "\n", img->slotclass);
 		if (img->variant)
-			g_string_append_printf(text, "\tVariant:   %s\n", img->variant);
+			g_string_append_printf(text, "    Variant:   %s\n", img->variant);
+		if (img->artifact)
+			g_string_append_printf(text, "    Artifact:  %s\n", img->artifact);
 		if (img->filename) {
 			g_autofree gchar* formatted_size = g_format_size_full(img->checksum.size, G_FORMAT_SIZE_LONG_FORMAT);
-			g_string_append_printf(text, "\tFilename:  %s\n", img->filename);
-			g_string_append_printf(text, "\tChecksum:  %s\n", img->checksum.digest);
-			g_string_append_printf(text, "\tSize:      %s\n", formatted_size);
+			g_string_append_printf(text, "    Filename:  %s\n", img->filename);
+			g_string_append_printf(text, "    Checksum:  %s\n", img->checksum.digest);
+			g_string_append_printf(text, "    Size:      %s\n", formatted_size);
 		} else {
-			g_string_append_printf(text, "\t(no image file)\n");
+			g_string_append_printf(text, "    (no image file)\n");
 		}
 
 		hooks = g_ptr_array_new();
@@ -1054,24 +1074,36 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		g_ptr_array_add(hooks, NULL);
 
 		temp_string = g_strjoinv(" ", (gchar**) hooks->pdata);
-		g_string_append_printf(text, "\tHooks:     %s\n", temp_string);
+		g_string_append_printf(text, "    Hooks:     %s\n", temp_string);
 		g_free(temp_string);
 
 		g_ptr_array_unref(hooks);
 
 		if (img->adaptive) {
 			temp_string = g_strjoinv(" ", (gchar**) img->adaptive);
-			g_string_append_printf(text, "\tAdaptive:  %s\n", temp_string);
+			g_string_append_printf(text, "    Adaptive:  %s\n", temp_string);
 			g_free(temp_string);
 		}
 
+		if (img->convert) {
+			temp_string = g_strjoinv(" ", (gchar**) img->convert);
+			g_string_append_printf(text, "    Convert:   %s\n", temp_string);
+			g_free(temp_string);
+		}
+		if (img->converted) {
+			g_string_append(text, "    Converted:");
+			for (guint i = 0; i < img->converted->len; i++) {
+				g_string_append_printf(text, " '%s'", (gchar*)img->converted->pdata[i]);
+			}
+			g_string_append_c(text, '\n');
+		}
+
 		cnt++;
 	}
 
 	return g_string_free(text, FALSE);
 }
 
-
 #if ENABLE_JSON
 /* Takes a GStrv and adds a JSON array to the builder. If the GStrv is NULL, an
  * empty array is added. */
@@ -1087,6 +1119,19 @@ static void strv_to_json_array(JsonBuilder *builder, GStrv strv)
 	}
 	json_builder_end_array(builder);
 }
+
+/* Takes a GPtrArray and adds a JSON array to the builder. */
+static void ptrarray_to_json_array(JsonBuilder *builder, GPtrArray *ptrarray)
+{
+	g_return_if_fail(JSON_IS_BUILDER(builder));
+	g_return_if_fail(ptrarray);
+
+	json_builder_begin_array(builder);
+	for (guint i = 0; i < ptrarray->len; i++) {
+		json_builder_add_string_value(builder, ptrarray->pdata[i]);
+	}
+	json_builder_end_array(builder);
+}
 #endif
 
 static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
@@ -1132,6 +1177,8 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 		json_builder_begin_object(builder);
 		json_builder_set_member_name(builder, img->slotclass);
 		json_builder_begin_object(builder);
+		json_builder_set_member_name(builder, "artifact");
+		json_builder_add_string_value(builder, img->artifact);
 		json_builder_set_member_name(builder, "variant");
 		json_builder_add_string_value(builder, img->variant);
 		json_builder_set_member_name(builder, "filename");
@@ -1154,6 +1201,14 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 		json_builder_end_array(builder);
 		json_builder_set_member_name(builder, "adaptive");
 		strv_to_json_array(builder, img->adaptive);
+		if (img->convert) {
+			json_builder_set_member_name(builder, "convert");
+			strv_to_json_array(builder, img->convert);
+		}
+		if (img->converted) {
+			json_builder_set_member_name(builder, "converted");
+			ptrarray_to_json_array(builder, img->converted);
+		}
 		json_builder_end_object(builder);
 		json_builder_end_object(builder);
 	}
@@ -1308,6 +1363,7 @@ typedef struct {
 	gchar *variant;
 	gchar *bootslot;
 	GHashTable *slots;
+	GVariant *artifacts;
 } RaucStatusPrint;
 
 static void free_status_print(RaucStatusPrint *status)
@@ -1320,6 +1376,8 @@ static void free_status_print(RaucStatusPrint *status)
 	g_free(status->bootslot);
 	if (status->slots)
 		g_hash_table_unref(status->slots);
+	if (status->artifacts)
+		g_variant_unref(status->artifacts);
 
 	g_free(status);
 }
@@ -1350,13 +1408,13 @@ static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint
 			slot->state == ST_BOOTED ? KBLD : "",
 			r_slot_slotstate_to_str(slot->state));
 	if (slot->bootname)
-		g_string_append_printf(text, "\n\tbootname: "KBLU "%s"KNRM, slot->bootname);
+		g_string_append_printf(text, "\n      bootname: "KBLU "%s"KNRM, slot->bootname);
 	if (slot->description)
-		g_string_append_printf(text, "\n\tdescription: %s", slot->description);
+		g_string_append_printf(text, "\n      description: %s", slot->description);
 	if (slot->mount_point)
-		g_string_append_printf(text, "\n\tmounted: %s", slot->mount_point);
+		g_string_append_printf(text, "\n      mounted: %s", slot->mount_point);
 	if (slot->bootname)
-		g_string_append_printf(text, "\n\tboot status: %s", slot->boot_good ? KGRN "good"KNRM : KRED "bad"KNRM);
+		g_string_append_printf(text, "\n      boot status: %s", slot->boot_good ? KGRN "good"KNRM : KRED "bad"KNRM);
 	if (status_detailed && slot_state) {
 		g_string_append_printf(text, "\n      slot status:");
 		g_string_append_printf(text, "\n          bundle:");
@@ -1394,6 +1452,74 @@ static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint
 	g_string_append_c(text, '\n');
 }
 
+static void r_string_append_repo(GString *text, GVariant *repo_var, RaucStatusPrint *status)
+{
+	const gchar *tmp = NULL;
+	if (g_variant_lookup(repo_var, "name", "&s", &tmp)) {
+		g_string_append_printf(text, KBLD "[%s]"KNRM " ", tmp);
+	}
+	if (g_variant_lookup(repo_var, "path", "&s", &tmp)) {
+		g_string_append_printf(text, "(%s)\n", tmp);
+	}
+	if (g_variant_lookup(repo_var, "type", "&s", &tmp)) {
+		g_string_append_printf(text, "  type: %s\n", tmp);
+	}
+
+	gboolean has_parent = FALSE;
+	if (g_variant_lookup(repo_var, "parent-class", "&s", &tmp)) {
+		g_string_append_printf(text, "  parent class: %s\n", tmp);
+		has_parent = TRUE;
+	}
+
+	g_autoptr(GVariant) artifacts_var = NULL;
+	if (g_variant_lookup(repo_var, "artifacts", "@aa{sv}", &artifacts_var)) {
+		gsize artifacts_n = g_variant_n_children(artifacts_var);
+		if (artifacts_n) {
+			g_string_append_printf(text, "  artifacts:\n");
+		} else {
+			g_string_append_printf(text, "  artifacts: (none)\n");
+		}
+
+		GVariantIter artifact_iter;
+		g_variant_iter_init(&artifact_iter, artifacts_var);
+		GVariant *artifact_var;
+		while (g_variant_iter_loop(&artifact_iter, "@a{sv}", &artifact_var)) {
+			if (g_variant_lookup(artifact_var, "name", "&s", &tmp)) {
+				g_string_append_printf(text, "    "KBLD "[%s]"KNRM "\n", tmp);
+			}
+			g_autoptr(GVariant) instances_var = NULL;
+			if (g_variant_lookup(artifact_var, "instances", "@aa{sv}", &instances_var)) {
+				GVariantIter digest_iter;
+				g_variant_iter_init(&digest_iter, instances_var);
+				GVariant *digest_var;
+				while (g_variant_iter_loop(&digest_iter, "@a{sv}", &digest_var)) {
+					if (g_variant_lookup(digest_var, "checksum", "&s", &tmp)) {
+						g_string_append_printf(text, "      checksum: %s\n", tmp);
+					}
+
+					g_autofree gchar **references = NULL;
+					if (g_variant_lookup(digest_var, "references", "^a&s", &references)) {
+						if (!has_parent) {
+							if (references[0]) {
+								g_string_append_printf(text, "        active\n");
+							} else {
+								g_string_append_printf(text, "        inactive\n");
+							}
+						} else {
+							g_autofree gchar *joined = g_strjoinv(" ", references);
+							if (references[0]) {
+								g_string_append_printf(text, "        references: '%s'\n", joined);
+							} else {
+								g_string_append_printf(text, "        references: (none)\n");
+							}
+						}
+					}
+				}
+			}
+		}
+	}
+}
+
 static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 {
 	GString *text = g_string_new(NULL);
@@ -1441,6 +1567,18 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 		}
 	}
 
+	if (status->artifacts && g_variant_n_children(status->artifacts) > 0) {
+		g_string_append(text, "=== Artifact Repo States ===\n");
+		GVariantIter repo_iter;
+		g_variant_iter_init(&repo_iter, status->artifacts);
+		GVariant *repo_var;
+		while (g_variant_iter_loop(&repo_iter, "@a{sv}", &repo_var)) {
+			r_string_append_repo(text, repo_var, status);
+
+			g_string_append(text, "\n");
+		}
+	}
+
 	return g_string_free(text, FALSE);
 }
 
@@ -1515,6 +1653,79 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 		}
 	}
 
+	gint repocnt = 0;
+	if (status->artifacts && g_variant_n_children(status->artifacts) > 0) {
+		GVariantIter repo_iter;
+		g_variant_iter_init(&repo_iter, status->artifacts);
+		GVariant *repo_var;
+		while (g_variant_iter_loop(&repo_iter, "@a{sv}", &repo_var)) {
+			repocnt++;
+
+			const gchar *tmp = NULL;
+			if (g_variant_lookup(repo_var, "name", "&s", &tmp)) {
+				r_ptr_array_add_printf(entries, "RAUC_REPO_NAME_%d=%s", repocnt, tmp);
+			}
+			if (g_variant_lookup(repo_var, "path", "&s", &tmp)) {
+				r_ptr_array_add_printf(entries, "RAUC_REPO_PATH_%d=%s", repocnt, tmp);
+			}
+			if (g_variant_lookup(repo_var, "type", "&s", &tmp)) {
+				r_ptr_array_add_printf(entries, "RAUC_REPO_TYPE_%d=%s", repocnt, tmp);
+			}
+
+			gboolean has_parent = FALSE;
+			if (g_variant_lookup(repo_var, "parent-class", "&s", &tmp)) {
+				r_ptr_array_add_printf(entries, "RAUC_REPO_PARENT_CLASS_%d=%s", repocnt, tmp);
+				has_parent = TRUE;
+			}
+
+			g_autoptr(GVariant) artifacts = NULL;
+			if (g_variant_lookup(repo_var, "artifacts", "@aa{sv}", &artifacts)) {
+				GVariantIter artifact_iter;
+				g_variant_iter_init(&artifact_iter, artifacts);
+				GVariant *artifact_var;
+				gint artifactcnt = 0;
+				while (g_variant_iter_loop(&artifact_iter, "@a{sv}", &artifact_var)) {
+					artifactcnt++;
+
+					if (g_variant_lookup(artifact_var, "name", "&s", &tmp)) {
+						r_ptr_array_add_printf(entries, "RAUC_REPO_ARTIFACT_NAME_%d_%d=%s", repocnt, artifactcnt, tmp);
+					}
+
+					g_autoptr(GVariant) instances_var = NULL;
+					if (g_variant_lookup(artifact_var, "instances", "@aa{sv}", &instances_var)) {
+						GVariantIter digest_iter;
+						g_variant_iter_init(&digest_iter, instances_var);
+						GVariant *digest_var;
+						gint instancecnt = 0;
+						while (g_variant_iter_loop(&digest_iter, "@a{sv}", &digest_var)) {
+							instancecnt++;
+
+							if (g_variant_lookup(digest_var, "checksum", "&s", &tmp)) {
+								r_ptr_array_add_printf(entries, "RAUC_REPO_ARTIFACT_INSTANCE_CHECKSUM_%d_%d_%d=%s",
+										repocnt, artifactcnt, instancecnt, tmp);
+							}
+
+							g_autofree gchar **references = NULL;
+							if (g_variant_lookup(digest_var, "references", "^a&s", &references)) {
+								if (!has_parent) {
+									r_ptr_array_add_printf(entries, "RAUC_REPO_ARTIFACT_INSTANCE_ACTIVE_%d_%d_%d=%s",
+											repocnt, artifactcnt, instancecnt, references[0] ? "1" : "0");
+								} else {
+									g_autofree gchar *joined = g_strjoinv(" ", references);
+									r_ptr_array_add_printf(entries, "RAUC_REPO_ARTIFACT_INSTANCE_REFERENCES_%d_%d_%d=%s",
+											repocnt, artifactcnt, instancecnt, joined);
+								}
+							}
+						}
+						formatter_shell_append_idx_str(entries, "RAUC_REPO_ARTIFACT_INSTANCES", repocnt, artifactcnt, instancecnt);
+					}
+				}
+				formatter_shell_append_idx_str(entries, "RAUC_REPO_ARTIFACTS", repocnt, -1, artifactcnt);
+			}
+		}
+	}
+	formatter_shell_append_idx_str(entries, "RAUC_REPOS", -1, -1, repocnt);
+
 	return r_ptr_array_env_to_shell(entries);
 }
 
@@ -1635,6 +1846,11 @@ static gchar* r_status_formatter_json(RaucStatusPrint *status, gboolean pretty)
 
 	json_builder_end_array(builder);
 
+	if (status->artifacts) {
+		json_builder_set_member_name(builder, "artifact-repositories");
+		json_builder_add_value(builder, json_gvariant_serialize(status->artifacts));
+	}
+
 	json_builder_end_object(builder);
 
 	gen = json_generator_new();
@@ -1851,6 +2067,17 @@ static gboolean retrieve_status_via_dbus(RaucStatusPrint **status_print, GError
 	if (primary)
 		istatus->primary = g_hash_table_lookup(istatus->slots, primary);
 
+	if (!r_installer_call_get_artifact_status_sync(proxy, &istatus->artifacts, NULL, &ierror)) {
+		if (g_dbus_error_is_remote_error(ierror))
+			g_dbus_error_strip_remote_error(ierror);
+		g_set_error(error,
+				G_IO_ERROR,
+				G_IO_ERROR_FAILED,
+				"error calling D-Bus method \"GetArtifactStatus\": %s", ierror->message);
+		g_error_free(ierror);
+		return FALSE;
+	}
+
 	*status_print = g_steal_pointer(&istatus);
 
 	return TRUE;
@@ -1859,7 +2086,6 @@ static gboolean retrieve_status_via_dbus(RaucStatusPrint **status_print, GError
 static gboolean print_status(RaucStatusPrint *status_print)
 {
 	g_autofree gchar *text = NULL;
-
 	if (!output_format || g_strcmp0(output_format, "readable") == 0) {
 		text = r_status_formatter_readable(status_print);
 	} else if (g_strcmp0(output_format, "shell") == 0) {
@@ -1873,6 +2099,7 @@ static gboolean print_status(RaucStatusPrint *status_print)
 		return FALSE;
 	}
 
+	g_strchomp(text);
 	g_print("%s\n", text);
 
 	return TRUE;
@@ -1916,9 +2143,16 @@ static gboolean status_start(int argc, char **argv)
 			while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot))
 				r_slot_status_load(slot);
 		}
-
 		status_print = g_new0(RaucStatusPrint, 1);
 
+		/* initialize without pruning, as an install might be ongoing in the background */
+		if (!r_artifacts_init(&ierror)) {
+			g_printerr("Failed to initialize artifact repos: %s\n", ierror->message);
+			g_clear_error(&ierror);
+			r_exit_status = 1;
+			return TRUE;
+		}
+
 		status_print->primary = r_boot_get_primary(&ierror);
 		if (!status_print->primary) {
 			g_printerr("Failed getting primary slot: %s\n", ierror->message);
@@ -1929,6 +2163,7 @@ static gboolean status_start(int argc, char **argv)
 		status_print->variant = g_strdup(r_context()->config->system_variant);
 		status_print->bootslot = g_strdup(r_context()->bootslot);
 		status_print->slots = g_hash_table_ref(r_context()->config->slots);
+		status_print->artifacts = r_artifacts_to_dict();
 	} else {
 		if (!retrieve_status_via_dbus(&status_print, &ierror)) {
 			g_printerr("Error retrieving slot status via D-Bus: %s\n",
@@ -2023,10 +2258,10 @@ static void r_event_log_booted(const RaucSlot *booted_slot)
 
 	g_return_if_fail(booted_slot);
 
-	message = g_strdup_printf("Booted into %s (%s)", booted_slot->name, booted_slot->bootname);
+	message = g_strdup_printf("Booted into %s (%s)", booted_slot->name, booted_slot->bootname ?: "");
 	fields[0].value = message;
 	fields[5].value = booted_slot->name;
-	fields[6].value = booted_slot->bootname;
+	fields[6].value = booted_slot->bootname ?: "";
 	fields[7].value = r_context()->boot_id;
 	if (booted_slot->status && booted_slot->status->bundle_hash) {
 		fields[8].value = booted_slot->status->bundle_hash;
@@ -2067,18 +2302,47 @@ static void create_run_links(void)
 	}
 
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
+	RaucSlot *booted_slot = NULL;
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
 		g_autofree gchar* path = NULL;
 
 		if (!(slot->state & ST_ACTIVE))
 			continue;
 
+		if (slot->state == ST_BOOTED)
+			booted_slot = slot;
+
 		path = g_build_filename("/run/rauc/slots/active", slot->sclass, NULL);
 
 		if (!r_update_symlink(slot->device, path, &ierror)) {
 			g_warning("Failed to create symlink for active slot: %s", ierror->message);
 		}
 	}
+
+	if (!booted_slot)
+		return;
+
+	if (g_mkdir_with_parents("/run/rauc/artifacts", 0755) != 0) {
+		g_warning("Failed to create /run/rauc/artifacts");
+		return;
+	}
+
+	g_hash_table_iter_init(&iter, r_context()->config->artifact_repos);
+	RArtifactRepo *repo;
+	while (g_hash_table_iter_next(&iter, NULL, (gpointer*)&repo)) {
+		g_autofree gchar* path = g_build_filename("/run/rauc/artifacts", repo->name, NULL);
+		g_autofree gchar* target = NULL;
+
+		if (!repo->parent_class) {
+			target = g_build_filename(repo->path, NULL);
+		} else {
+			target = g_build_filename(repo->path, booted_slot->name, NULL);
+		}
+
+		if (!r_update_symlink(target, path, &ierror)) {
+			g_warning("Failed to create symlink for artifact repository: %s", ierror->message);
+		}
+	}
 }
 
 G_GNUC_UNUSED
@@ -2094,6 +2358,13 @@ static gboolean service_start(int argc, char **argv)
 		return TRUE;
 	}
 
+	if (!r_artifacts_init(&ierror)) {
+		g_printerr("Failed to initialize artifact repos: %s\n", ierror->message);
+		g_clear_error(&ierror);
+		r_exit_status = 1;
+		return TRUE;
+	}
+
 	create_run_links();
 
 	if (r_context()->system_status) {
@@ -2375,6 +2646,34 @@ static void create_option_groups(void)
 	g_option_group_add_entries(service_group, entries_service);
 }
 
+// Callback function to handle the repeated -C option
+static gboolean collect_config_values(const gchar *option_name, const gchar *value, gpointer data, GError **error)
+{
+	if (value == NULL) {
+		g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Error in parsing override: you must specify it in format SECTION:KEY=VALUE");
+		return FALSE;
+	}
+
+	g_auto(GStrv) key_value_split = g_strsplit(value, "=", 2);
+	if (g_strv_length(key_value_split) != 2) {
+		g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Error in parsing override: Missing '='");
+		return FALSE;
+	}
+
+	g_auto(GStrv) section_key_split = g_strsplit(key_value_split[0], ":", 2);
+	if (g_strv_length(section_key_split) != 2) {
+		g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Error in parsing override: Missing ':'");
+		return FALSE;
+	}
+
+	ConfigFileOverride *override = g_new(ConfigFileOverride, 1);
+	override->section = g_steal_pointer(&section_key_split[0]);
+	override->name = g_steal_pointer(&section_key_split[1]);
+	override->value = g_steal_pointer(&key_value_split[1]);
+	r_context_conf()->configoverride = g_list_append(r_context_conf()->configoverride, override);
+	return TRUE;
+}
+
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
@@ -2383,6 +2682,7 @@ static void cmdline_handler(int argc, char **argv)
 	g_autoptr(GOptionContext) context = NULL;
 	GOptionEntry entries[] = {
 		{"conf", 'c', 0, G_OPTION_ARG_FILENAME, &confpath, "config file", "FILENAME"},
+		{"confopt", 'C', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, collect_config_values, "config settings that override parameters from the config file", "SECTION:KEY=VALUE" },
 		/* NOTE: cert and key kept for backwards-compatibility, but made invisible */
 		{"cert", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &certpath, "cert file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 		{"key", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &keypath, "key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
@@ -2433,9 +2733,9 @@ static void cmdline_handler(int argc, char **argv)
 		{STATUS, "status", "status",
 		 "Show system status\n\n"
 		 "List of status commands (default slot is the currently booted slot):\n"
-		 "  mark-good [booted | other | <SLOT_NAME>] \tMark the slot as good\n"
-		 "  mark-bad [booted | other | <SLOT_NAME>] \tMark the slot as bad\n"
-		 "  mark-active [booted | other | <SLOT_NAME>] \tMark the slot as active",
+		 "  mark-good [booted | other | <SLOT_NAME>]    Mark the slot as good\n"
+		 "  mark-bad [booted | other | <SLOT_NAME>]     Mark the slot as bad\n"
+		 "  mark-active [booted | other | <SLOT_NAME>]  Mark the slot as active",
 		 status_start, status_group, R_CONTEXT_CONFIG_MODE_REQUIRED, TRUE},
 		{WRITE_SLOT, "write-slot", "write-slot <SLOTNAME> <IMAGE>",
 		 "Write image to slot and bypass all update logic",
@@ -2463,29 +2763,29 @@ static void cmdline_handler(int argc, char **argv)
 			"\n"
 			"List of rauc bundle handling commands:\n"
 #if ENABLE_CREATE == 1
-			"  bundle\t\tCreate a bundle\n"
-			"  resign\t\tResign an already signed bundle\n"
-			"  convert\t\tConvert classic to casync bundle\n"
-			"  encrypt\t\tEncrypt a crypt bundle\n"
-			"  replace-signature\tReplaces the signature of an already signed bundle\n"
-			"  extract-signature\tExtract the bundle signature\n"
+			"  bundle                  Create a bundle\n"
+			"  resign                  Resign an already signed bundle\n"
+			"  convert                 Convert classic to casync bundle\n"
+			"  encrypt                 Encrypt a crypt bundle\n"
+			"  replace-signature       Replaces the signature of an already signed bundle\n"
+			"  extract-signature       Extract the bundle signature\n"
 #endif
-			"  extract\t\tExtract the bundle content\n"
-			"  info\t\t\tShow bundle information\n"
+			"  extract                 Extract the bundle content\n"
+			"  info                    Show bundle information\n"
 			"\n"
 			"List of rauc target commands:\n"
 #if ENABLE_SERVICE == 1
-			"  service\t\tStart RAUC service\n"
+			"  service                 Start RAUC service\n"
 #endif
-			"  install\t\tInstall a bundle\n"
-			"  status\t\tShow status\n"
-			"  mount\t\t\tMount a bundle\n"
-			"  write-slot\t\tWrite image to slot and bypass all update logic\n"
+			"  install                 Install a bundle\n"
+			"  status                  Show status\n"
+			"  mount                   Mount a bundle\n"
+			"  write-slot              Write image to slot and bypass all update logic\n"
 			"\n"
 			"Environment variables:\n"
-			"  RAUC_KEY_PASSPHRASE Passphrase to use for accessing key files (signing only)\n"
-			"  RAUC_PKCS11_MODULE  Library filename for PKCS#11 module (signing only)\n"
-			"  RAUC_PKCS11_PIN     PIN to use for accessing PKCS#11 keys (signing only)");
+			"  RAUC_KEY_PASSPHRASE     Passphrase to use for accessing key files (signing only)\n"
+			"  RAUC_PKCS11_MODULE      Library filename for PKCS#11 module (signing only)\n"
+			"  RAUC_PKCS11_PIN         PIN to use for accessing PKCS#11 keys (signing only)");
 
 	if (!g_option_context_parse(context, &argc, &argv, &error)) {
 		g_printerr("%s\n", error->message);
