@@ -22,6 +22,7 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 	gchar *value;
 	g_auto(GStrv) hooks = NULL;
 	gsize entries;
+	g_auto(GStrv) converted = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
 
@@ -30,16 +31,28 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 	g_return_val_if_fail(image == NULL || *image == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
+	/* We support several formats:
+	 * - [image.rootfs]
+	 * - [image.rootfs.product-a]
+	 * - [image.appfs/app-1]
+	 * - [image.appfs/app-1.product-a]
+	 */
+
 	groupsplit = g_strsplit(group, ".", 3);
 	g_assert_cmpint(g_strv_length(groupsplit), >=, 2);
+	g_assert_cmpstr(groupsplit[0], ==, "image");
 
-	iimage->slotclass = g_strdup(groupsplit[1]);
+	g_auto(GStrv) targetsplit = NULL;
+	targetsplit = g_strsplit(groupsplit[1], "/", 2);
+	iimage->slotclass = g_strdup(targetsplit[0]);
 
-	/* if we have a variant part in group, store it here */
+	/* Do we have an artifact name for this image? */
+	if (g_strv_length(targetsplit) == 2)
+		iimage->artifact = g_strdup(targetsplit[1]);
+
+	/* Do we have a variant name for this image? */
 	if (g_strv_length(groupsplit) == 3)
 		iimage->variant = g_strdup(groupsplit[2]);
-	else
-		iimage->variant = NULL;
 
 	value = key_file_consume_string(key_file, group, "sha256", NULL);
 	if (value) {
@@ -92,6 +105,16 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 	iimage->adaptive = g_key_file_get_string_list(key_file, group, "adaptive", NULL, NULL);
 	g_key_file_remove_key(key_file, group, "adaptive", NULL);
 
+	iimage->convert = g_key_file_get_string_list(key_file, group, "convert", NULL, NULL);
+	g_key_file_remove_key(key_file, group, "convert", NULL);
+
+	converted = g_key_file_get_string_list(key_file, group, "converted", NULL, NULL);
+	g_key_file_remove_key(key_file, group, "converted", NULL);
+	if (converted) {
+		iimage->converted = g_ptr_array_new_with_free_func(g_free);
+		r_ptr_array_addv(iimage->converted, converted, TRUE);
+	}
+
 	if (!check_remaining_keys(key_file, group, &ierror)) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -425,6 +448,36 @@ out:
 	return res;
 }
 
+static gboolean check_manifest_plain(const RaucManifest *mf, GError **error)
+{
+	g_return_val_if_fail(mf, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	g_assert(mf->bundle_format == R_MANIFEST_FORMAT_PLAIN);
+
+	for (GList *elem = mf->images; elem != NULL; elem = elem->next) {
+		RaucImage *image = elem->data;
+
+		/* Check for features not supported in plain bundles */
+		if (image->artifact) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Artifacts are not supported in plain bundles");
+			return FALSE;
+		}
+		if (image->convert || (image->converted && image->converted->len)) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Image converters are not supported in plain bundles");
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+/**
+ * Check a loaded manifest for consistency. Manifests generated by 'rauc bundle'
+ * should pass this check if they are compatible with the running version.
+ *
+ * This function is called for both internal and external manifests.
+ */
 static gboolean check_manifest_bundled(const RaucManifest *mf, GError **error)
 {
 	for (GList *l = mf->images; l != NULL; l = l->next) {
@@ -460,6 +513,77 @@ static gboolean check_manifest_bundled(const RaucManifest *mf, GError **error)
 				return FALSE;
 			}
 		}
+
+		if (image->convert) {
+			guint expected_len = g_strv_length(image->convert);
+
+			if (!image->converted) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing converted outputs for image %s", image->filename);
+				return FALSE;
+			}
+
+			if (expected_len != image->converted->len) {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Inconsistent number of converted inputs/outputs for image %s", image->filename);
+				return FALSE;
+			}
+		}
+	}
+
+	return TRUE;
+}
+
+gboolean check_manifest_input(const RaucManifest *mf, GError **error)
+{
+	GError *ierror = NULL;
+
+	if (!check_manifest_common(mf, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
+	switch (mf->bundle_format) {
+		case R_MANIFEST_FORMAT_PLAIN:
+		case R_MANIFEST_FORMAT_CRYPT:
+		case R_MANIFEST_FORMAT_VERITY:
+			break;
+		default: {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported bundle format in input manifest");
+			return FALSE;
+		}
+	}
+
+	if (mf->bundle_format == R_MANIFEST_FORMAT_PLAIN) {
+		if (!check_manifest_plain(mf, &ierror)) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+	}
+
+	for (GList *l = mf->images; l != NULL; l = l->next) {
+		RaucImage *image = l->data;
+
+		g_assert(image);
+
+		if (image->filename && strchr(image->filename, '/')) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR,
+					"Image filename %s must not contain '/'", image->filename);
+			return FALSE;
+		}
+		if (image->checksum.digest) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR,
+					"Unexpected digest for image %s in input manifest", image->filename);
+			return FALSE;
+		}
+		if (image->checksum.size != -1) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR,
+					"Unexpected size %"G_GOFFSET_FORMAT " for image %s in input manifest", image->checksum.size, image->filename);
+			return FALSE;
+		}
+		if (image->converted && image->converted->len) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR,
+					"Unexpected 'converted' option in input manifest");
+			return FALSE;
+		}
 	}
 
 	return TRUE;
@@ -493,6 +617,11 @@ gboolean check_manifest_internal(const RaucManifest *mf, GError **error)
 		goto out;
 	}
 
+	if (!check_manifest_plain(mf, &ierror)) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	if (mf->bundle_crypt_key) {
 		g_set_error_literal(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unexpected key for crypt bundle in internal manifest");
 		goto out;
@@ -610,6 +739,23 @@ out:
 	return res;
 }
 
+gboolean check_manifest_create(const RaucManifest *mf, GError **error)
+{
+	for (GList *l = mf->images; l != NULL; l = l->next) {
+		RaucImage *image = l->data;
+
+		g_assert(image);
+
+		if (image->hooks.install && (image->hooks.pre_install || image->hooks.post_install)) {
+			g_set_error_literal(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR,
+					"An 'install' hook must not be combined with 'pre-install' or 'post-install' hooks");
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 static GKeyFile *prepare_manifest(const RaucManifest *mf)
 {
 	g_autoptr(GKeyFile) key_file = NULL;
@@ -686,6 +832,12 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 
 		group = g_strconcat(RAUC_IMAGE_PREFIX ".", image->slotclass, NULL);
 
+		if (image->artifact) {
+			gchar *tmp = group;
+			group = g_strconcat(group, "/", image->artifact, NULL);
+			g_free(tmp);
+		}
+
 		if (image->variant) {
 			gchar *tmp = group;
 			group = g_strconcat(group, ".", image->variant, NULL);
@@ -719,6 +871,13 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 		if (image->adaptive)
 			g_key_file_set_string_list(key_file, group, "adaptive",
 					(const gchar * const *)image->adaptive, g_strv_length(image->adaptive));
+
+		if (image->convert)
+			g_key_file_set_string_list(key_file, group, "convert",
+					(const gchar * const *)image->convert, g_strv_length(image->convert));
+		if (image->converted && image->converted->len)
+			g_key_file_set_string_list(key_file, group, "converted",
+					(const gchar * const *)image->converted->pdata, image->converted->len);
 	}
 
 	if (mf->meta) {
@@ -842,6 +1001,8 @@ GVariant* r_manifest_to_dict(const RaucManifest *manifest)
 
 		g_variant_builder_open(&builder, G_VARIANT_TYPE("a{sv}"));
 		g_variant_builder_add(&builder, "{sv}", "slot-class", g_variant_new_string(img->slotclass));
+		if (img->artifact)
+			g_variant_builder_add(&builder, "{sv}", "artifact", g_variant_new_string(img->artifact));
 		if (img->variant)
 			g_variant_builder_add(&builder, "{sv}", "variant", g_variant_new_string(img->variant));
 		if (img->filename)
@@ -862,6 +1023,11 @@ GVariant* r_manifest_to_dict(const RaucManifest *manifest)
 		if (img->adaptive)
 			g_variant_builder_add(&builder, "{sv}", "adaptive", g_variant_new_strv((const gchar * const*)(img->adaptive), -1));
 
+		if (img->convert)
+			g_variant_builder_add(&builder, "{sv}", "convert", g_variant_new_strv((const gchar * const*)(img->convert), -1));
+		if (img->converted)
+			g_variant_builder_add(&builder, "{sv}", "converted", g_variant_new_strv((const gchar * const*)(img->converted->pdata), img->converted->len));
+
 		g_variant_builder_close(&builder);
 	}
 	g_variant_dict_insert(&root_dict, "images", "v", g_variant_builder_end(&builder));
@@ -893,6 +1059,34 @@ GVariant* r_manifest_to_dict(const RaucManifest *manifest)
 	return g_variant_dict_end(&root_dict);
 }
 
+gboolean r_manifest_has_artifact_image(const RaucManifest *manifest, const gchar *repo, const gchar *artifact)
+{
+	g_return_val_if_fail(manifest != NULL, FALSE);
+	g_return_val_if_fail((!repo && !artifact) ||
+			(repo && !artifact) ||
+			(repo && artifact), FALSE);
+
+	for (GList *l = manifest->images; l != NULL; l = l->next) {
+		const RaucImage *img = l->data;
+
+		/* skip images which are not an artifact */
+		if (!img->artifact)
+			continue;
+
+		/* skip images for different repos */
+		if (repo && g_strcmp0(repo, img->slotclass) != 0)
+			continue;
+
+		/* skip images with different artifact names */
+		if (artifact && g_strcmp0(artifact, img->artifact) != 0)
+			continue;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 RaucImage *r_new_image(void)
 {
 	RaucImage *image = g_new0(RaucImage, 1);
@@ -910,10 +1104,13 @@ void r_free_image(gpointer data)
 		return;
 
 	g_free(image->slotclass);
+	g_free(image->artifact);
 	g_free(image->variant);
 	g_free(image->checksum.digest);
 	g_free(image->filename);
 	g_strfreev(image->adaptive);
+	g_strfreev(image->convert);
+	g_clear_pointer(&image->converted, (GDestroyNotify)g_ptr_array_unref);
 	g_free(image);
 }
 
