@@ -374,7 +374,6 @@ static gboolean write_boot_switch_partition(RaucImage *image, const gchar *devic
 		const struct boot_switch_partition *dest_partition,
 		gsize len_header_last,
 		GError **error)
-
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -570,7 +569,7 @@ static gboolean casync_extract_image(RaucImage *image, gchar *dest, int out_fd,
 		goto extract;
 	}
 
-	if (g_str_has_suffix(image->filename, ".caidx" )) {
+	if (g_str_has_suffix(image->filename, ".caidx")) {
 		/* We need to have the seed slot (bind) mounted to a distinct
 		 * path to allow seeding. E.g. using mount path '/' for the
 		 * rootfs slot seed is inaproppriate as it contains virtual
@@ -856,7 +855,6 @@ static gboolean copy_block_hash_index_image_to_dev(RaucImage *image, RaucSlot *s
 		goto out;
 	}
 
-
 	/* Flush to block device before closing to assure content is written to disk */
 	if (fsync(target_fd) == -1) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED, "Syncing content to slot failed: %s", strerror(errno));
@@ -1287,9 +1285,9 @@ out:
 
 static gboolean unpack_archive(RaucImage *image, gchar *dest, GError **error)
 {
-	if (g_str_has_suffix(image->filename, ".caidx" ))
+	if (g_str_has_suffix(image->filename, ".caidx"))
 		return casync_extract_image(image, dest, -1, error);
-	else if (g_str_has_suffix(image->filename, ".catar" ))
+	else if (g_str_has_suffix(image->filename, ".catar"))
 		return casync_extract_image(image, dest, -1, error);
 	else
 		return untar_image(image, dest, error);
@@ -1302,6 +1300,7 @@ static gboolean unpack_archive(RaucImage *image, gchar *dest, GError **error)
  * @param hook_cmd first argument to the hook script
  * @param image image to be installed (optional)
  * @param slot target slot
+ * @param variables extra environment variables, or NULL
  * @param error return location for a GError, or NULL
  *
  * @return TRUE on success, FALSE if an error occurred
@@ -2209,21 +2208,17 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 		g_propagate_error(error, ierror);
 		goto out;
 	}
-	switch (part_active) {
-		case -1:
-			part_active_str = g_strdup("<none>");
-			break;
-		case 6:
-			part_active_str = g_strdup(realdev);
-			break;
-		default:
-			part_active_str = g_strdup_printf("%sboot%d", realdev, part_active);
+
+	if (part_active == -1) {
+		g_warning("eMMC device was not enabled for booting, yet. Ignoring.");
+		/* For simplicity: Consider boot1 to be active */
+		part_active = 1;
+	} else {
+		g_message("Found active eMMC boot partition %sboot%d", realdev, part_active);
 	}
-	g_message("Found active eMMC boot partition %s", part_active_str);
 
 	/* create a temporary RaucSlot with the actual (currently inactive) boot
-	 * partition as device; for simplicity reasons: in case the user partition
-	 * is active use mmcblkXboot1, in case no partition is active use mmcblkXboot0
+	 * partition as device.
 	 */
 	part_slot = g_new0(RaucSlot, 1);
 	part_slot->device = g_strdup_printf(
