@@ -9,6 +9,11 @@ typedef struct {
 	gboolean custom_handler;
 	gboolean hooks;
 	gboolean slots;
+
+	const gchar *artifact_file;
+	const gchar *artifact_slotclass;
+	const gchar *artifact_convert;
+
 	RManifestBundleFormat format;
 	gboolean no_bundle_version;
 	const gchar *bundle_version;
@@ -16,6 +21,7 @@ typedef struct {
 
 typedef struct {
 	const gchar *min_bundle_version;
+	gboolean artifact_repos;
 } SystemTestOptions;
 
 guint8* random_bytes(gsize size, guint32 seed);
