@@ -98,7 +98,12 @@ device=/dev/appfs-1\n\
 type=ext4\n\
 extra-mkfs-opts=-L mylabel -i 8192\n\
 parent=rootfs.1\n\
-install-same=false\n";
+install-same=false\n\
+[artifacts.containers]\n\
+path=/var/artifacts/containers\n\
+type=trees\n\
+";
+
 	g_autofree gchar* pathname = write_tmp_file(fixture->tmpdir, "full_config.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -256,7 +261,6 @@ mountprefix=/mnt/myrauc/\n";
 compatible=FooCorp Super BarBazzer\n\
 mountprefix=/mnt/myrauc/\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", boot_inval_cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -311,7 +315,6 @@ boot-attempts=5\n\
 boot-attempts-primary=-10\n\
 mountprefix=/mnt/myrauc/\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "valid_bootloader.conf", valid_boot_attempts_cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -366,7 +369,6 @@ device=/dev/null\n\
 type=oups\n\
 \t";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "system.conf", invalid_slot_type, NULL);
 	g_assert_nonnull(pathname);
 
@@ -393,7 +395,6 @@ device=/dev/null\n\
 parent=invalid\n\
 \t";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "nonexisting_bootloader.conf", nonexisting_parent, NULL);
 	g_assert_nonnull(pathname);
 
@@ -524,6 +525,35 @@ bootname=theslot\n";
 	g_clear_error(&ierror);
 }
 
+static void config_file_duplicate_slotclass(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	g_autoptr(RaucConfig) config = NULL;
+	GError *ierror = NULL;
+	g_autofree gchar* pathname = NULL;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/null\n\
+bootname=theslot\n\
+\n\
+[artifacts.rootfs]\n\
+path=/var/artifacts/containers\n\
+type=trees\n\
+";
+
+	pathname = write_tmp_file(fixture->tmpdir, "duplicate_bootname.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_DUPLICATE_CLASS);
+	g_assert_cmpstr(ierror->message, ==, "Existing slot class 'rootfs' cannot be used as artifact repo name!");
+	g_clear_error(&ierror);
+}
+
 static void config_file_typo(ConfigFileFixture *fixture, const gchar *cfg_file)
 {
 	g_autoptr(RaucConfig) config = NULL;
@@ -746,7 +776,6 @@ bootloader=barebox\n\
 mountprefix=/mnt/myrauc/\n\
 activate-installed=true\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -772,7 +801,6 @@ bootloader=barebox\n\
 mountprefix=/mnt/myrauc/\n\
 activate-installed=false\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -908,7 +936,6 @@ activate-installed=false\n\
 [slot.rootfs.0]\n\
 device=/dev/null\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "extra_mount.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -917,13 +944,11 @@ device=/dev/null\n";
 	g_assert_true(res);
 	g_assert_nonnull(config);
 
-
 	slot = g_hash_table_lookup(config->slots, "rootfs.0");
 	g_assert_nonnull(slot);
 	g_assert_cmpstr(slot->extra_mount_opts, ==, NULL);
 }
 
-
 static void config_file_extra_mount_opts(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
@@ -944,7 +969,6 @@ activate-installed=false\n\
 device=/dev/null\n\
 extra-mount-opts=ro,noatime\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "extra_mount.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -1009,7 +1033,6 @@ compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
 mountprefix=/mnt/myrauc/\n";
 
-
 	pathname = write_tmp_file(fixture->tmpdir, "valid_bootloader.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -1568,6 +1591,9 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/duplicate-bootname", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_duplicate_bootname,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/duplicate-slotclass", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_duplicate_slotclass,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/typo-in-boolean-allow-mounted-key", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_typo_in_boolean_allow_mounted_key,
 			config_file_fixture_tear_down);
