@@ -271,9 +271,16 @@ device=/path/to/prebootloader";
 static void install_fixture_tear_down(InstallFixture *fixture,
 		gconstpointer user_data)
 {
+	InstallData *data = (InstallData*) user_data;
+
 	if (!fixture->tmpdir)
 		return;
 
+	if (data && data->manifest_test_options.artifact_file) {
+		g_autofree gchar *tree_cmd = g_strdup_printf("tree -a %s/repos", fixture->tmpdir);
+		system(tree_cmd);
+	}
+
 	test_umount(fixture->tmpdir, "slot");
 	test_umount(fixture->tmpdir, "bootloader");
 	test_rm_tree(fixture->tmpdir, "");
@@ -295,7 +302,6 @@ static void install_test_target(InstallFixture *fixture,
 	g_autoptr(GError) error = NULL;
 	gboolean result;
 
-
 	const gchar *manifest_file = "\
 [update]\n\
 compatible=FooCorp Super BarBazzer\n\
@@ -913,7 +919,6 @@ readonly=true\n\
 	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
 }
 
-
 static void test_install_image_variants(void)
 {
 	g_autofree gchar *tmpdir = NULL;
@@ -926,7 +931,7 @@ static void test_install_image_variants(void)
 	g_autoptr(GError) error = NULL;
 	gboolean res;
 
-#define MANIFEST_VARIANT "\
+	const gchar *manifest_variant = "\
 [update]\n\
 compatible=foo\n\
 \n\
@@ -935,24 +940,21 @@ filename=dummy.img\n\
 \n\
 [image.rootfs]\n\
 filename=dummy.img\n\
-"
-
-#define MANIFEST_DEFAULT_VARIANT "\
+";
+	const gchar *manifest_default_variant = "\
 [update]\n\
 compatible=foo\n\
 \n\
 [image.rootfs]\n\
 filename=dummy.img\n\
-"
-
-#define MANIFEST_OTHER_VARIANT "\
+";
+	const gchar *manifest_other_variant = "\
 [update]\n\
 compatible=foo\n\
 \n\
 [image.rootfs.variant-2]\n\
 filename=dummy.img\n\
-"
-
+";
 	const gchar *system_conf_variant = "\
 [system]\n\
 compatible=foo\n\
@@ -986,7 +988,7 @@ device=/dev/null\n\
 	g_assert_nonnull(tgrp);
 
 	/* Test with manifest containing default and specific variant */
-	data = g_bytes_new_static(MANIFEST_VARIANT, sizeof(MANIFEST_VARIANT));
+	data = g_bytes_new_static(manifest_variant, strlen(manifest_variant));
 	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
@@ -1007,7 +1009,7 @@ device=/dev/null\n\
 	g_clear_pointer(&install_plans, g_ptr_array_unref);
 
 	/* Test with manifest containing only default variant */
-	data = g_bytes_new_static(MANIFEST_DEFAULT_VARIANT, sizeof(MANIFEST_DEFAULT_VARIANT));
+	data = g_bytes_new_static(manifest_default_variant, strlen(manifest_default_variant));
 	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
@@ -1028,7 +1030,7 @@ device=/dev/null\n\
 	g_clear_pointer(&install_plans, g_ptr_array_unref);
 
 	/* Test with manifest containing only non-matching specific variant (must fail) */
-	data = g_bytes_new_static(MANIFEST_OTHER_VARIANT, sizeof(MANIFEST_OTHER_VARIANT));
+	data = g_bytes_new_static(manifest_other_variant, strlen(manifest_other_variant));
 	res = load_manifest_mem(data, &rm, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
@@ -1126,15 +1128,16 @@ static void install_test_bundle(InstallFixture *fixture,
 	if (!data->install_err_domain) {
 		g_assert_no_error(ierror);
 		g_assert_true(res);
-
-		slotfile = g_build_filename(fixture->tmpdir, "images/rootfs-1", NULL);
-		mountdir = g_build_filename(fixture->tmpdir, "mnt", NULL);
-		g_assert(test_mkdir_relative(fixture->tmpdir, "mnt", 0777) == 0);
-		testfilepath = g_build_filename(mountdir, "verify.txt", NULL);
-		g_assert(test_mount(slotfile, mountdir));
-		g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
-		g_assert(test_umount(fixture->tmpdir, "mnt"));
-		g_assert(test_rm_tree(fixture->tmpdir, "mnt"));
+		if (data->manifest_test_options.slots) {
+			slotfile = g_build_filename(fixture->tmpdir, "images/rootfs-1", NULL);
+			mountdir = g_build_filename(fixture->tmpdir, "mnt", NULL);
+			g_assert(test_mkdir_relative(fixture->tmpdir, "mnt", 0777) == 0);
+			testfilepath = g_build_filename(mountdir, "verify.txt", NULL);
+			g_assert(test_mount(slotfile, mountdir));
+			g_assert(g_file_test(testfilepath, G_FILE_TEST_IS_REGULAR));
+			g_assert(test_umount(fixture->tmpdir, "mnt"));
+			g_assert(test_rm_tree(fixture->tmpdir, "mnt"));
+		}
 	} else {
 		/* expected installation error */
 		g_assert_error(ierror, data->install_err_domain, data->install_err_code);
@@ -1224,7 +1227,7 @@ static void install_test_bundle_hook_install_check(InstallFixture *fixture,
 	args->notify = install_notify;
 	args->cleanup = install_cleanup;
 	g_assert_false(do_install_bundle(args, &ierror));
-	g_assert_cmpstr(ierror->message, ==, "Bundle rejected: Hook returned: No, I won't install this!");
+	g_assert_cmpstr(ierror->message, ==, "Bundle rejected: Hook returned: 'No, I won't install this!'");
 
 	args->status_result = 0;
 	args->cleanup(args);
@@ -1519,8 +1522,8 @@ int main(int argc, char *argv[])
 
 		install_data = dup_test_data(ptrs, (&(InstallData) {
 			.manifest_test_options = {
-			        .format = format,
-			        .slots = TRUE,
+				.format = format,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/bootname/%s", format_name),
@@ -1558,9 +1561,9 @@ int main(int argc, char *argv[])
 				NULL,
 			})),
 			.manifest_test_options = {
-			        .custom_handler = TRUE,
-			        .hooks = FALSE,
-			        .slots = TRUE,
+				.custom_handler = TRUE,
+				.hooks = FALSE,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/bundle-custom-handler/%s", format_name),
@@ -1571,9 +1574,9 @@ int main(int argc, char *argv[])
 		install_data = dup_test_data(ptrs, (&(InstallData) {
 			.message_needles = NULL,
 			.manifest_test_options = {
-			        .custom_handler = FALSE,
-			        .hooks = TRUE,
-			        .slots = TRUE,
+				.custom_handler = FALSE,
+				.hooks = TRUE,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/bundle-hook/install-check/%s", format_name),
@@ -1584,9 +1587,9 @@ int main(int argc, char *argv[])
 		install_data = dup_test_data(ptrs, (&(InstallData) {
 			.message_needles = NULL,
 			.manifest_test_options = {
-			        .custom_handler = FALSE,
-			        .hooks = TRUE,
-			        .slots = TRUE,
+				.custom_handler = FALSE,
+				.hooks = TRUE,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/bundle-hook/slot-install/%s", format_name),
@@ -1597,9 +1600,9 @@ int main(int argc, char *argv[])
 		install_data = dup_test_data(ptrs, (&(InstallData) {
 			.message_needles = NULL,
 			.manifest_test_options = {
-			        .custom_handler = FALSE,
-			        .hooks = TRUE,
-			        .slots = TRUE,
+				.custom_handler = FALSE,
+				.hooks = TRUE,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/bundle-hook/slot-post-install/%s", format_name),
@@ -1610,9 +1613,9 @@ int main(int argc, char *argv[])
 		install_data = dup_test_data(ptrs, (&(InstallData) {
 			.message_needles = NULL,
 			.manifest_test_options = {
-			        .custom_handler = TRUE,
-			        .hooks = TRUE,
-			        .slots = TRUE,
+				.custom_handler = TRUE,
+				.hooks = TRUE,
+				.slots = TRUE,
 			},
 		}));
 		g_test_add(dup_test_printf(ptrs, "/install/already-mounted/%s", format_name),
@@ -1623,8 +1626,8 @@ int main(int argc, char *argv[])
 
 	install_data = dup_test_data(ptrs, (&(InstallData) {
 		.manifest_test_options = {
-		        .format = R_MANIFEST_FORMAT_VERITY,
-		        .slots = TRUE,
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.slots = TRUE,
 		},
 	}));
 	g_test_add("/install/adaptive",
@@ -1641,12 +1644,12 @@ int main(int argc, char *argv[])
 		.install_err_domain = R_INSTALL_ERROR,
 		.install_err_code = R_INSTALL_ERROR_VERSION_MISMATCH,
 		.system_test_options = {
-		        .min_bundle_version = "1.0",
+			.min_bundle_version = "1.0",
 		},
 		.manifest_test_options = {
-		        .format = R_MANIFEST_FORMAT_VERITY,
-		        .slots = TRUE,
-		        .bundle_version = "0.0.1",
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.slots = TRUE,
+			.bundle_version = "0.0.1",
 		},
 	}));
 	g_test_add("/install/min-bundle-version/version-bad",
@@ -1657,12 +1660,12 @@ int main(int argc, char *argv[])
 
 	install_data = dup_test_data(ptrs, (&(InstallData) {
 		.system_test_options = {
-		        .min_bundle_version = "1.0",
+			.min_bundle_version = "1.0",
 		},
 		.manifest_test_options = {
-		        .format = R_MANIFEST_FORMAT_VERITY,
-		        .slots = TRUE,
-		        .bundle_version = "1.0.1",
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.slots = TRUE,
+			.bundle_version = "1.0.1",
 		},
 	}));
 	g_test_add("/install/min-bundle-version/version-good-1",
@@ -1673,12 +1676,12 @@ int main(int argc, char *argv[])
 
 	install_data = dup_test_data(ptrs, (&(InstallData) {
 		.system_test_options = {
-		        .min_bundle_version = "1.0",
+			.min_bundle_version = "1.0",
 		},
 		.manifest_test_options = {
-		        .format = R_MANIFEST_FORMAT_VERITY,
-		        .slots = TRUE,
-		        .bundle_version = "2025.1-foo+bar",
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.slots = TRUE,
+			.bundle_version = "2025.1-foo+bar",
 		},
 	}));
 	g_test_add("/install/min-bundle-version/version-good-2",
@@ -1691,12 +1694,12 @@ int main(int argc, char *argv[])
 		.install_err_domain = R_INSTALL_ERROR,
 		.install_err_code = R_INSTALL_ERROR_VERSION_MISMATCH,
 		.system_test_options = {
-		        .min_bundle_version = "1.0",
+			.min_bundle_version = "1.0",
 		},
 		.manifest_test_options = {
-		        .format = R_MANIFEST_FORMAT_VERITY,
-		        .slots = TRUE,
-		        .no_bundle_version = TRUE,
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.slots = TRUE,
+			.no_bundle_version = TRUE,
 		},
 	}));
 	g_test_add("/install/min-bundle-version/version-missing",
@@ -1705,5 +1708,82 @@ int main(int argc, char *argv[])
 			install_test_bundle,
 			install_fixture_tear_down);
 
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+			.artifact_repos = TRUE,
+		},
+		.manifest_test_options = {
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.artifact_file = "artifact-1.file",
+			.artifact_slotclass = "files",
+		},
+	}));
+	g_test_add("/install/artifacts/file",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle, install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+			.artifact_repos = TRUE,
+		},
+		.manifest_test_options = {
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.artifact_file = "payload-common.tar",
+			.artifact_slotclass = "trees",
+		},
+	}));
+	g_test_add("/install/artifacts/common-tar/direct",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle, install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+			.artifact_repos = TRUE,
+		},
+		.manifest_test_options = {
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.artifact_file = "payload-common.tar",
+			.artifact_slotclass = "trees",
+			.artifact_convert = "tar-extract",
+		},
+	}));
+	g_test_add("/install/artifacts/common-tar/extracted",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle, install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+			.artifact_repos = TRUE,
+		},
+		.manifest_test_options = {
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.artifact_file = "payload-special.tar",
+			.artifact_slotclass = "trees",
+		},
+	}));
+	g_test_add("/install/artifacts/special-tar/direct",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle, install_test_bundle,
+			install_fixture_tear_down);
+
+	install_data = dup_test_data(ptrs, (&(InstallData) {
+		.system_test_options = {
+			.artifact_repos = TRUE,
+		},
+		.manifest_test_options = {
+			.format = R_MANIFEST_FORMAT_VERITY,
+			.artifact_file = "payload-special.tar",
+			.artifact_slotclass = "trees",
+			.artifact_convert = "tar-extract",
+		},
+	}));
+	g_test_add("/install/artifacts/special-tar/extracted",
+			InstallFixture, install_data,
+			install_fixture_set_up_bundle, install_test_bundle,
+			install_fixture_tear_down);
+
 	return g_test_run();
 }
